from django.utils.translation import gettext_lazy as _

from mayan.apps.rest_api import serializers
from mayan.apps.user_management.serializers import UserSerializer

from ..models import Notification

from .event_serializers import EventSerializer


class NotificationSerializer(serializers.HyperlinkedModelSerializer):
    action = EventSerializer(
        label=_(message='Action'), read_only=True
    )
    user = UserSerializer(
        label=_(message='User'), read_only=True
    )

    class Meta:
        extra_kwargs = {
            'url': {
                'label': _(message='URL'),
                'lookup_url_kwarg': 'notification_id',
                'view_name': 'rest_api:notification-detail'
            }
        }
        fields = ('action', 'read', 'url', 'user')
        model = Notification
        read_only_fields = ('action', 'url', 'user')



from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('document_states', '0038_alter_workflowinstance_state_active')
    ]

    operations = [
        migrations.AddField(
            field=models.BooleanField(
                default=False, help_text='Ignore workflow instances if '
                'they are in their final state.',
                verbose_name='Ignore completed'
            ), model_name='workflow', name='ignore_completed'
        )
    ]



from django.core.exceptions import ValidationError as DjangoValidationError
from django.utils.translation import gettext_lazy as _

from rest_framework.exceptions import ValidationError
from rest_framework.reverse import reverse

from mayan.apps.documents.serializers.document_serializers import (
    DocumentSerializer
)
from mayan.apps.documents.serializers.document_type_serializers import (
    DocumentTypeSerializer
)
from mayan.apps.rest_api import serializers
from mayan.apps.rest_api.relations import (
    FilteredPrimaryKeyRelatedField, FilteredSimplePrimaryKeyRelatedField
)

from .models.document_type_metadata_type_models import (
    DocumentTypeMetadataType
)
from .models.metadata_instance_models import DocumentMetadata
from .models.metadata_type_models import MetadataType
from .permissions import (
    permission_document_metadata_add, permission_metadata_type_edit
)


class MetadataTypeSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        extra_kwargs = {
            'url': {
                'label': _(message='URL'), 'lookup_field': 'pk',
                'lookup_url_kwarg': 'metadata_type_id',
                'view_name': 'rest_api:metadatatype-detail'
            }
        }
        fields = (
            'default', 'id', 'label', 'lookup', 'name', 'parser',
            'parser_arguments', 'url', 'validation', 'validation_arguments'
        )
        model = MetadataType
        read_only_fields = ('id', 'url')


class DocumentTypeMetadataTypeSerializer(
    serializers.HyperlinkedModelSerializer
):
    document_type = DocumentTypeSerializer(
        label=_(message='Document type'), read_only=True
    )
    metadata_type = MetadataTypeSerializer(
        label=_(message='Metadata type'), read_only=True
    )
    metadata_type_id = FilteredSimplePrimaryKeyRelatedField(
        help_text=_(
            message='Primary key of the metadata type to be added.'
        ), label=_(message='Metadata type ID'), source_model=MetadataType,
        source_permission=permission_metadata_type_edit, write_only=True
    )
    url = serializers.SerializerMethodField(
        label=_(message='URL')
    )

    class Meta:
        create_only_fields = ('metadata_type_id',)
        fields = (
            'document_type', 'id', 'metadata_type', 'metadata_type_id',
            'required', 'url'
        )
        model = DocumentTypeMetadataType
        read_only_field = ('document_type', 'id', 'metadata_type', 'url')

    def get_url(self, instance):
        return reverse(
            format=self.context['format'], kwargs={
                'document_type_id': instance.document_type.pk,
                'metadata_type_id': instance.pk
            }, request=self.context['request'],
            viewname='rest_api:documenttypemetadatatype-detail'
        )

    def validate(self, attrs):
        if self.instance:
            return attrs

        attrs['document_type'] = self.context['external_object']
        attrs['metadata_type'] = MetadataType.objects.get(
            pk=attrs.pop('metadata_type_id')
        )

        instance = DocumentTypeMetadataType(**attrs)
        try:
            instance.full_clean()
        except DjangoValidationError as exception:
            raise ValidationError(detail=exception)

        return attrs


class DocumentMetadataSerializer(
    serializers.ModelSerializer
):
    metadata_type_id = FilteredPrimaryKeyRelatedField(
        help_text=_(
            message='Primary key of the metadata type to be added to the '
            'document.'
        ), label=_(message='Metadata type ID'), source_model=MetadataType,
        source_permission=permission_document_metadata_add, write_only=True
    )
    document = DocumentSerializer(
        label=_(message='Document'), read_only=True
    )
    metadata_type = MetadataTypeSerializer(
        label=_(message='Metadata type'), read_only=True
    )
    url = serializers.SerializerMethodField(
        label=_(message='URL')
    )

    class Meta:
        create_only_fields = ('metadata_type_id',)
        fields = (
            'document', 'id', 'metadata_type', 'metadata_type_id', 'url',
            'value'
        )
        model = DocumentMetadata
        read_only_fields = ('document', 'id', 'metadata_type', 'url')

    def get_url(self, instance):
        return reverse(
            format=self.context['format'], kwargs={
                'document_id': instance.document.pk,
                'metadata_id': instance.pk
            }, request=self.context['request'],
            viewname='rest_api:documentmetadata-detail'
        )

    def validate(self, attrs):
        if self.instance:
            self.instance.value = attrs['value']

            try:
                self.instance.full_clean()
            except DjangoValidationError as exception:
                raise ValidationError(detail=exception)

            attrs['value'] = self.instance.value

            return attrs
        else:
            attrs['document'] = self.context['external_object']
            attrs['metadata_type'] = MetadataType.objects.get(
                pk=attrs.pop('metadata_type_id').pk
            )

            instance = DocumentMetadata(**attrs)
            try:
                instance.full_clean()
            except DjangoValidationError as exception:
                raise ValidationError(detail=exception)

            attrs['value'] = instance.value

            return attrs



from functools import cache
import logging

from django.core.exceptions import ImproperlyConfigured
from django.template import RequestContext, Variable, VariableDoesNotExist

from .class_mixins import TemplateObjectMixin
from .links import ResolvedLink
from .utils import get_current_view_name

logger = logging.getLogger(name=__name__)


class Menu(TemplateObjectMixin):
    _registry = {}

    @staticmethod
    def get_result_label(item):
        """
        Method to help sort results by label.
        """
        if isinstance(item, ResolvedLink):
            return str(item.link.text)
        else:
            return str(item.label)

    @classmethod
    def get(cls, name):
        return cls._registry[name]

    @classmethod
    def remove(cls, name):
        del cls._registry[name]

    def __init__(
        self, name, cache_class_associations=True, condition=None, icon=None,
        label=None, non_sorted_sources=None, title=None
    ):
        if name in self.__class__._registry:
            raise Exception('A menu with this name already exists')

        self.cache_class_associations = cache_class_associations
        self.bound_links = {}
        self.condition = condition
        self.excluded_links = {}
        self._icon = icon
        self.label = label
        self.link_positions = {}
        self.name = name
        self.non_sorted_sources = non_sorted_sources or []
        self.proxy_exclusions = set()
        self.title = title
        self.unbound_links = {}
        self.__class__._registry[name] = self

    def __repr__(self):
        return '<Menu: {}>'.format(self.name)

    def _map_links_to_source(
        self, links, source, map_variable, position=None
    ):
        source_links = getattr(self, map_variable).setdefault(
            source, []
        )

        position = position or len(source_links)

        for link_index, link in enumerate(iterable=links):
            source_links.append(link)
            self.link_positions[link] = position + link_index

    def add_proxy_exclusion(self, source):
        # Avoid this source proxy model from binding to all links in the menu.
        self.proxy_exclusions.add(source)

    def add_unsorted_source(self, source):
        self.non_sorted_sources.append(source)

    def bind_links(self, links, exclude=None, sources=None, position=None):
        """
        Associate a link to a model, a view inside this menu.
        - exclude: Avoid a proxy model from resolving the bound links.
        """
        try:
            for source in sources:
                self._map_links_to_source(
                    links=links, map_variable='bound_links',
                    position=position, source=source
                )
            for source in exclude:
                self._map_links_to_source(
                    links=links, map_variable='excluded_links',
                    position=position, source=source
                )
        except TypeError:
            # Links without a source are always displayed.
            self._map_links_to_source(
                links=links, map_variable='bound_links',
                position=position, source=sources
            )

    def get_icon(self, context):
        return self._icon

    def do_matched_links_update(
        self, matched_links, bound_object, unbound_object, excluded_object
    ):
        # Add bound links that match the object.
        # Remove unbound links that match the object.
        # Remove unbound global unbound links.
        # Remove excluded links that match the object.

        matched_links.update(
            set(
                self.bound_links.get(
                    bound_object, ()
                )
            ) - set(
                self.unbound_links.get(
                    unbound_object, ()
                )
            ) - set(
                self.unbound_links.get(
                    None, ()
                )
            ) - set(
                self.excluded_links.get(
                    excluded_object, ()
                )
            )
        )

        return matched_links

    @cache
    def get_links_for_class_cached(self, resolved_navigation_object_class):
        return self.get_links_for_class_non_cached(
            resolved_navigation_object_class=resolved_navigation_object_class
        )

    def get_links_for_class_non_cached(self, resolved_navigation_object_class):
        matched_links = set()

        try:
            mro = resolved_navigation_object_class.__mro__
        except AttributeError:
            # Not a class, direct instance match.
            bound_object = resolved_navigation_object_class
            excluded_object = resolved_navigation_object_class
            unbound_object = resolved_navigation_object_class

            matched_links = self.do_matched_links_update(
                bound_object=bound_object, excluded_object=excluded_object,
                matched_links=matched_links, unbound_object=unbound_object
            )
        else:
            # Get proxy results.
            # Remove the results explicitly excluded.
            # Execute after the root model results to allow
            # a proxy to override an existing results.
            model = resolved_navigation_object_class
            try:
                proxy_parent_model = model._meta.proxy_for_model
            except AttributeError:
                """It's not a model, treat as a generic class."""

                for super_class in mro[:-1]:
                    bound_object = super_class
                    excluded_object = resolved_navigation_object_class
                    unbound_object = super_class

                    matched_links = self.do_matched_links_update(
                        bound_object=bound_object,
                        excluded_object=excluded_object,
                        matched_links=matched_links,
                        unbound_object=unbound_object
                    )
            else:
                # It is a model.
                matched_links = self.do_matched_links_update(
                    bound_object=model,
                    excluded_object=model,
                    matched_links=matched_links,
                    unbound_object=model
                )

                if proxy_parent_model:
                    # It is a model proxy. Add parent links except for
                    # menu exclusions and bind excludes.
                    if model not in self.proxy_exclusions:
                        matched_links = self.do_matched_links_update(
                            bound_object=proxy_parent_model,
                            excluded_object=model,
                            matched_links=matched_links,
                            unbound_object=proxy_parent_model
                        )

        return matched_links

    def get_navigation_object_class(self, resolved_navigation_object):
        if resolved_navigation_object is None:
            # None means "always on" menu links.
            return None
        elif isinstance(resolved_navigation_object, str):
            # It is a view name, return the value.
            return resolved_navigation_object
        else:
            try:
                # Try it as a queryset.
                resolved_navigation_object.query
            except AttributeError:
                try:
                    # Try it as a list.
                    item = resolved_navigation_object[0]
                except TypeError:
                    try:
                        # Try as a model instance or model.
                        model = resolved_navigation_object._meta.model
                    except AttributeError:
                        # Must be a non model class instance, return the class.
                        return resolved_navigation_object.__class__
                    else:
                        return model
                else:
                    # It is a list, return the class of the first item.
                    return self.get_navigation_object_class(
                        resolved_navigation_object=item
                    )
            else:
                # It is a queryset, return the model.
                model = resolved_navigation_object.model

                return self.get_navigation_object_class(
                    resolved_navigation_object=model
                )

    def get_resolved_navigation_object_list(self, context, source):
        resolved_navigation_object_list = []

        if source:
            resolved_navigation_object_list = [source]
        else:
            navigation_object_list = context.get(
                'navigation_object_list', ('object',)
            )

            logger.debug(
                'navigation_object_list: %s', navigation_object_list
            )

            # Multiple objects
            for navigation_object in navigation_object_list:
                try:
                    resolved_variable = Variable(
                        var=navigation_object
                    ).resolve(context=context)
                except VariableDoesNotExist:
                    """Non fatal. Proceed with next variable name in list."""
                else:
                    resolved_navigation_object_list.append(
                        resolved_variable
                    )

        logger.debug(
            'resolved_navigation_object_list: %s', str(
                resolved_navigation_object_list
            )
        )
        return resolved_navigation_object_list

    def get_result_position(self, item):
        """
        Method to help sort results by position.
        """
        if isinstance(item, ResolvedLink):
            return self.link_positions.get(item.link, 0)
        else:
            return self.link_positions.get(item, 0) or 0

    def resolve(
        self, context=None, request=None, source=None, sort_results=False
    ):
        result = []

        if self.cache_class_associations:
            function_get_links_for_class = self.get_links_for_class_cached
        else:
            function_get_links_for_class = self.get_links_for_class_non_cached

        if not context and not request:
            raise ImproperlyConfigured(
                'Must provide a context or a request in order to resolve '
                'the menu.'
            )

        if not context:
            context = RequestContext(request=request)

        if not self.check_condition(context=context):
            return result

        try:
            request = self.get_request(context=context, request=request)
        except VariableDoesNotExist:
            # Cannot resolve any menus without a request object.
            # Return an empty list.
            return result

        current_view_name = get_current_view_name(request=request)
        if not current_view_name:
            return result

        resolved_navigation_object_list = self.get_resolved_navigation_object_list(
            context=context, source=source
        )

        for resolved_navigation_object in resolved_navigation_object_list:
            navigation_object_class = self.get_navigation_object_class(
                resolved_navigation_object=resolved_navigation_object
            )
            matched_links = function_get_links_for_class(
                resolved_navigation_object_class=navigation_object_class
            )

            result.extend(
                self.resolve_matched_links(
                    context=context,
                    matched_links=matched_links,
                    resolved_navigation_object=resolved_navigation_object,
                    as_resolved_object=True
                )
            )

        # Resolve view links.
        navigation_object_class = self.get_navigation_object_class(
            resolved_navigation_object=current_view_name
        )
        matched_links = function_get_links_for_class(
            resolved_navigation_object_class=navigation_object_class
        )

        result.extend(
            self.resolve_matched_links(
                context=context,
                matched_links=matched_links,
                resolved_navigation_object=current_view_name
            )
        )

        # Resolve "always one" menu links.
        navigation_object_class = self.get_navigation_object_class(
            resolved_navigation_object=None
        )
        matched_links = function_get_links_for_class(
            resolved_navigation_object_class=navigation_object_class
        )

        result.extend(
            self.resolve_matched_links(
                context=context,
                matched_links=matched_links,
                resolved_navigation_object=None
            )
        )

        # Sort links.
        if result:
            unsorted_source = False
            for resolved_navigation_object in resolved_navigation_object_list:
                for source in self.non_sorted_sources:
                    if isinstance(resolved_navigation_object, source):
                        unsorted_source = True
                        break

            if sort_results and not unsorted_source:
                for link_group in result:
                    link_group['links'].sort(key=Menu.get_result_label)
            else:
                for link_group in result:
                    link_group['links'].sort(key=self.get_result_position)

        return result

    def resolve_matched_links(
        self, context, matched_links, resolved_navigation_object,
        as_resolved_object=False
    ):
        result = []

        object_resolved_links = []

        for link in matched_links:
            kwargs = {'context': context}

            if as_resolved_object:
                kwargs['resolved_object'] = resolved_navigation_object

            if isinstance(link, Menu):
                condition = link.check_condition(**kwargs)
                if condition:
                    object_resolved_links.append(link)
            else:
                # "Always show" links.
                resolved_link = link.resolve(**kwargs)
                if resolved_link:
                    object_resolved_links.append(resolved_link)

        if object_resolved_links:
            result.append(
                {
                    'links': object_resolved_links,
                    'object': resolved_navigation_object
                }
            )

        return result

    def unbind_links(self, links, sources=None):
        """
        Allow unbinding links from sources. Used to allow 3rd party apps to
        change the link binding of core apps without changing the core apps.
        """
        if sources is None:
            # Unsourced links display always.
            self._map_links_to_source(
                links=links, source=None, map_variable='unbound_links'
            )
        else:
            for source in sources:
                self._map_links_to_source(
                    links=links, source=source, map_variable='unbound_links'
                )

        self.get_links_for_class_cached.cache_clear()



from django.utils.translation import gettext_lazy as _

from mayan.apps.common.apps import MayanAppConfig


class CredentialsGoogleApp(MayanAppConfig):
    app_namespace = 'credentials_google'
    name = 'mayan.apps.credentials_google'
    verbose_name = _('Credentials Google')



from django.utils.translation import gettext_lazy as _

from mayan.apps.dependencies.classes import BinaryDependency

from .drivers import FileMetadataDriverEXIF

arguments = FileMetadataDriverEXIF.get_argument_values_from_settings()

BinaryDependency(
    help_text=_(
        message='Library and program to read and write meta information in '
        'multimedia files.'
    ), module=__name__, name='exiftool', path=arguments['exiftool_path']
)



from django.db import migrations


def operation_rename_credential_modules(apps, schema_editor):
    StoredCredential = apps.get_model(
        app_label='credentials', model_name='StoredCredential'
    )

    for stored_credential in StoredCredential.objects.using(alias=schema_editor.connection.alias).all():
        parts = stored_credential.backend_path.split('.')
        parts[-2] = 'credential_backends'
        stored_credential.backend_path = '.'.join(parts)
        stored_credential.save()


def operation_rename_credential_modules_reverse(apps, schema_editor):
    StoredCredential = apps.get_model(
        app_label='credentials', model_name='StoredCredential'
    )

    for stored_credential in StoredCredential.objects.using(alias=schema_editor.connection.alias).all():
        parts = stored_credential.backend_path.split('.')
        parts[-2] = 'credentials'
        stored_credential.backend_path = '.'.join(parts)
        stored_credential.save()


class Migration(migrations.Migration):
    dependencies = [
        ('credentials', '0001_initial')
    ]

    operations = [
        migrations.RunPython(
            code=operation_rename_credential_modules,
            reverse_code=operation_rename_credential_modules_reverse
        )
    ]



from django.utils.translation import gettext_lazy as _

from mayan.apps.icons.icons import Icon
from mayan.apps.source_compressed.source_backends.mixins import (
    SourceBackendMixinCompressed
)
from mayan.apps.source_interactive.source_backends.mixins import (
    SourceBackendMixinInteractive
)
from mayan.apps.source_stored_files.source_backends.filesystem_source_mixins import (
    SourceBackendMixinStoredFileLocationFilesystem
)
from mayan.apps.source_stored_files.source_backends.stored_file_source_mixins import (
    SourceBackendMixinStoredFileInteractive
)
from mayan.apps.sources.source_backends.base import SourceBackend
from mayan.apps.sources.source_backends.mixins import (
    SourceBackendMixinRegularExpression
)


class SourceBackendStagingFolder(
    SourceBackendMixinCompressed,
    SourceBackendMixinStoredFileLocationFilesystem,
    SourceBackendMixinInteractive, SourceBackendMixinRegularExpression,
    SourceBackendMixinStoredFileInteractive, SourceBackend
):
    icon = Icon(driver_name='fontawesome', symbol='file')
    label = _(message='Staging folder')



from django.utils.translation import gettext_lazy as _

from mayan.apps.smart_settings.settings import setting_cluster

from .literals import (
    DEFAULT_SEARCH_BACKEND, DEFAULT_SEARCH_BACKEND_ARGUMENTS,
    DEFAULT_SEARCH_DEFAULT_OPERATOR, DEFAULT_SEARCH_DISABLE_SIMPLE_SEARCH,
    DEFAULT_SEARCH_INDEXING_CHUNK_SIZE,
    DEFAULT_SEARCH_MATCH_ALL_DEFAULT_VALUE,
    DEFAULT_SEARCH_MODEL_FIELD_DISABLE, DEFAULT_SEARCH_QUERY_RESULTS_LIMIT,
    DEFAULT_SEARCH_QUERY_RESULTS_LIMIT_ERROR, DEFAULT_SEARCH_RESULTS_LIMIT,
    DEFAULT_SEARCH_SAVED_RESULTSET_RESULTS_LIMIT,
    DEFAULT_SEARCH_SAVED_RESULTSETS_PER_USER_LIMIT,
    DEFAULT_SEARCH_SAVED_RESULTSET_TIME_TO_LIVE,
    DEFAULT_SEARCH_SAVED_RESULTSET_TIME_TO_LIVE_INCREMENT,
    DEFAULT_SEARCH_STORE_RESULTS_DEFAULT_VALUE, SCOPE_OPERATOR_CHOICES
)

setting_namespace = setting_cluster.do_namespace_add(
    label=_(message='Search'), name='search'
)

setting_backend = setting_namespace.do_setting_add(
    default=DEFAULT_SEARCH_BACKEND, global_name='SEARCH_BACKEND', help_text=_(
        message='Full path to the backend to be used to handle the search.'
    )
)
setting_backend_arguments = setting_namespace.do_setting_add(
    default=DEFAULT_SEARCH_BACKEND_ARGUMENTS,
    global_name='SEARCH_BACKEND_ARGUMENTS', help_text=_(
        message='Arguments to pass to the search backend. For example values '
        'to change the behavior, host names, or authentication arguments.'
    )
)
setting_default_operator = setting_namespace.do_setting_add(
    choices=SCOPE_OPERATOR_CHOICES.keys(),
    global_name='SEARCH_DEFAULT_OPERATOR',
    default=DEFAULT_SEARCH_DEFAULT_OPERATOR, help_text=_(
        message='The search operator to use when none is specified.'
    )
)
setting_disable_simple_search = setting_namespace.do_setting_add(
    choices=('false', 'true'),
    default=DEFAULT_SEARCH_DISABLE_SIMPLE_SEARCH,
    global_name='SEARCH_DISABLE_SIMPLE_SEARCH', help_text=_(
        message='Disables the single term bar search leaving only the '
        'advanced search button.'
    )
)
setting_indexing_chunk_size = setting_namespace.do_setting_add(
    default=DEFAULT_SEARCH_INDEXING_CHUNK_SIZE,
    global_name='SEARCH_INDEXING_CHUNK_SIZE', help_text=_(
        message='Amount of objects to process when performing bulk indexing.'
    )
)
setting_match_all_default_value = setting_namespace.do_setting_add(
    global_name='SEARCH_MATCH_ALL_DEFAULT_VALUE',
    default=DEFAULT_SEARCH_MATCH_ALL_DEFAULT_VALUE, help_text=_(
        message='Sets the default state of the "Match all" checkbox.'
    )
)
setting_query_results_limit = setting_namespace.do_setting_add(
    default=DEFAULT_SEARCH_QUERY_RESULTS_LIMIT,
    global_name='SEARCH_QUERY_RESULTS_LIMIT', help_text=_(
        message='Maximum number of search results to fetch and display per '
        'search query unit.'
    )
)
setting_query_results_limit_error = setting_namespace.do_setting_add(
    default=DEFAULT_SEARCH_QUERY_RESULTS_LIMIT_ERROR,
    global_name='SEARCH_QUERY_RESULTS_LIMIT_ERROR', help_text=_(
        message='Raise an error when the number of search results '
        'exceed the value of `SEARCH_QUERY_RESULTS_LIMIT`.'
    )
)
setting_results_limit = setting_namespace.do_setting_add(
    default=DEFAULT_SEARCH_RESULTS_LIMIT, global_name='SEARCH_RESULTS_LIMIT',
    help_text=_(
        message='Maximum number of search results to fetch and display.'
    )
)
setting_saved_resultsets_per_user_limit = setting_namespace.do_setting_add(
    default=DEFAULT_SEARCH_SAVED_RESULTSETS_PER_USER_LIMIT,
    global_name='SEARCH_SAVED_RESULTSETS_PER_USER_LIMIT', help_text=_(
        message='Maximum number of saved resultsets to keep per user.'
    )
)
setting_saved_resultset_results_limit = setting_namespace.do_setting_add(
    default=DEFAULT_SEARCH_SAVED_RESULTSET_RESULTS_LIMIT,
    global_name='SEARCH_SAVED_RESULTSET_RESULTS_LIMIT', help_text=_(
        message='Maximum number of results to store per resultset.'
    )
)
setting_saved_resultset_time_to_live = setting_namespace.do_setting_add(
    default=DEFAULT_SEARCH_SAVED_RESULTSET_TIME_TO_LIVE,
    global_name='SEARCH_SAVED_RESULTSET_TIME_TO_LIVE', help_text=_(
        message='Time to keep the resultset in seconds.'
    )
)
setting_saved_resultset_time_to_live_increment = setting_namespace.do_setting_add(
    default=DEFAULT_SEARCH_SAVED_RESULTSET_TIME_TO_LIVE_INCREMENT,
    global_name='SEARCH_SAVED_RESULTSET_TIME_TO_LIVE_INCREMENT', help_text=_(
        message='Amount to increase the time to live on each access of the '
        'resultset.'
    )
)
setting_search_model_field_disable = setting_namespace.do_setting_add(
    global_name='SEARCH_MODEL_FIELD_DISABLE',
    default=DEFAULT_SEARCH_MODEL_FIELD_DISABLE, help_text=_(
        message='Specifies the fields from which search model are to be '
        'disabled. The format is a dictionary of lists. The search model '
        'name is the dictionary key and the value is a list of the full '
        'search field name. Disabled fields will neither be available for '
        'search nor be indexed by the search backend.'
    )
)
setting_store_results_default_value = setting_namespace.do_setting_add(
    global_name='SEARCH_STORE_RESULTS_DEFAULT_VALUE',
    default=DEFAULT_SEARCH_STORE_RESULTS_DEFAULT_VALUE, help_text=_(
        message='Sets the default state of the "Store results" checkbox.'
    )
)



from django.utils.translation import gettext_lazy as _

from mayan.apps.events.classes import EventTypeNamespace

namespace = EventTypeNamespace(
    label=_(message='Document comments'), name='document_comments'
)

event_document_comment_created = namespace.add_event_type(
    label=_(message='Document comment created'), name='create'
)
event_document_comment_deleted = namespace.add_event_type(
    label=_(message='Document comment deleted'), name='delete'
)
event_document_comment_edited = namespace.add_event_type(
    label=_(message='Document comment edited'), name='edited'
)



from django.template import Library
from django.utils.module_loading import import_string

register = Library()


@register.simple_tag(name='icons_get_icon')
def tag_icons_get_icon(icon_path, **kwargs):
    extra_context = {}

    for key, value in kwargs.items():
        if '__' in key:
            subdictionary = extra_context
            parts = key.split('__')
            for part in parts:
                subdictionary.setdefault(
                    part, {}
                )
                dictionary_pointer = subdictionary
                subdictionary = subdictionary[part]

            dictionary_pointer[part] = value
        else:
            extra_context[key] = value

    icon_class = import_string(dotted_path=icon_path)
    return icon_class.render(**extra_context)


@register.simple_tag(name='icons_icon_render')
def tag_icons_icon_render(icon, enable_shadow=False):
    return icon.render(
        extra_context={'enable_shadow': enable_shadow}
    )



from datetime import timedelta

from django.utils.translation import gettext_lazy as _

from mayan.apps.task_manager.classes import CeleryQueue
from mayan.apps.task_manager.workers import worker_a, worker_c

from .settings import setting_event_prune_task_interval

queue_events_fast = CeleryQueue(
    label=_(message='Events fast'), name='events_fast', worker=worker_a
)
queue_events_slow = CeleryQueue(
    label=_(message='Events slow'), name='events_slow', transient=True,
    worker=worker_c
)

queue_events_fast.add_task_type(
    dotted_path='mayan.apps.events.tasks.task_event_commit',
    label=_(message='Commit an event'), name='task_event_commit'
)

queue_events_slow.add_task_type(
    dotted_path='mayan.apps.events.tasks.task_event_prune',
    label=_(message='Launch the event prune backend'),
    name='task_event_prune',
    schedule=timedelta(seconds=setting_event_prune_task_interval.value),
)
queue_events_slow.add_task_type(
    dotted_path='mayan.apps.events.tasks.task_event_queryset_clear',
    label=_(message='Clear event querysets'),
    name='task_event_queryset_clear'
)
queue_events_slow.add_task_type(
    dotted_path='mayan.apps.events.tasks.task_event_queryset_export',
    label=_(message='Export event querysets'),
    name='task_event_queryset_export'
)



import re

from django.template import TemplateSyntaxError
from django.utils.module_loading import import_string


def get_tag_name(tag):
    cached_name = getattr(tag, '_cached_name', None)
    if cached_name:
        return cached_name
    else:
        module_name = getattr(tag, '__module__', None)

        dotted_path = '{}.register'.format(module_name)

        register = import_string(dotted_path=dotted_path)

        tag_dict = {}
        tag_dict.update(
            getattr(
                register, 'filters', {}
            )
        )
        tag_dict.update(
            getattr(
                register, 'tags', {}
            )
        )

        tag_name = None

        for name, function in tag_dict.items():
            if function is tag:
                tag_name = name
                break

            closure = getattr(function, '__closure__', None)
            if closure:
                for cell in closure:
                    if cell.cell_contents is tag:
                        tag_name = name
                        break

            wrapped = getattr(function, '__wrapped__', None)
            if wrapped:
                closure = getattr(wrapped, '__closure__', None)
                if closure:
                    for cell in closure:
                        if cell.cell_contents is tag:
                            tag_name = name
                            break

        tag._cached_name = tag_name or tag.__name__

        return tag._cached_name


def process_regex_flags(**kwargs):
    result = 0

    REGEX_FLAGS = {
        'ascii': re.ASCII,
        'ignorecase': re.IGNORECASE,
        'locale': re.LOCALE,
        'multiline': re.MULTILINE,
        'dotall': re.DOTALL,
        'verbose': re.VERBOSE
    }

    for key, value in kwargs.items():
        if value is True:
            try:
                result = result | REGEX_FLAGS[key]
            except KeyError:
                raise TemplateSyntaxError(
                    'Unknown or unsupported regular expression '
                    'flag: "{}"'.format(key)
                )

    return result



from django.utils.translation import gettext_lazy as _

from mayan.apps.rest_api import serializers
from mayan.apps.user_management.serializers import UserSerializer

from ..models.recently_accessed_document_models import (
    RecentlyAccessedDocument
)

from .document_serializers import DocumentSerializer


class RecentlyAccessedDocumentSerializer(serializers.ModelSerializer):
    document = DocumentSerializer(
        label=_(message='Document'), read_only=True
    )
    user = UserSerializer(
        label=_(message='User'), read_only=True
    )

    class Meta:
        fields = ('document', 'id', 'datetime_accessed', 'user')
        model = RecentlyAccessedDocument
        read_only_fields = ('document', 'id', 'datetime_accessed', 'user')



from django.utils.translation import gettext_lazy as _

from mayan.apps.authentication.link_conditions import (
    condition_user_is_authenticated
)
from mayan.apps.navigation.links import Link

from .icons import (
    icon_user_confirm_property_view, icon_user_confirm_property_view_delete,
    icon_user_view_modes
)

link_user_confirm_view_property_delete_multiple = Link(
    icon=icon_user_confirm_property_view_delete,
    tags='dangerous', text=_(message='Delete'),
    view='views:user_views_confirm_delete_multiple'
)
link_user_confirm_view_property_delete_single = Link(
    kwargs={'user_confirmation_view_id': 'resolved_object.pk'},
    icon=icon_user_confirm_property_view_delete,
    tags='dangerous', text=_(message='Delete'),
    view='views:user_views_confirm_delete_single'
)
link_user_confirm_view_property_list = Link(
    condition=condition_user_is_authenticated,
    kwargs={'user_id': 'resolved_object.pk'},
    icon=icon_user_confirm_property_view, text=_(message='Confirm views'),
    view='views:user_views_confirm_list'
)
link_user_view_modes = Link(
    condition=condition_user_is_authenticated,
    kwargs={'user_id': 'resolved_object.pk'}, icon=icon_user_view_modes,
    text=_(message='View modes'), view='views:user_view_modes'
)



from django.shortcuts import reverse
from django.template import RequestContext
from django.utils.translation import gettext_lazy as _

from mayan.apps.acls.models import AccessControlList
from mayan.apps.documents.models.document_models import Document
from mayan.apps.views.generics import MultipleObjectFormActionView
from mayan.apps.views.view_mixins import ExternalObjectViewMixin

from ..forms import TagMultipleSelectionForm
from ..icons import (
    icon_document_tag_list, icon_document_tag_multiple_attach,
    icon_document_tag_multiple_remove

)
from ..links import link_document_tag_multiple_attach
from ..models import DocumentTag, Tag
from ..permissions import (
    permission_tag_attach, permission_tag_remove, permission_tag_view
)

from .tag_views import TagListView


class TagAttachActionView(MultipleObjectFormActionView):
    form_class = TagMultipleSelectionForm
    object_permission = permission_tag_attach
    pk_url_kwarg = 'document_id'
    source_queryset = Document.valid.all()
    success_message_plural = _(
        message='Tags attached to %(count)d documents successfully.'
    )
    success_message_single = _(
        message='Tags attached to document "%(object)s" successfully.'
    )
    success_message_singular = _(
        message='Tags attached to %(count)d document successfully.'
    )
    title_plural = _(message='Attach tags to %(count)d documents.')
    title_single = _(message='Attach tags to document: %(object)s')
    title_singular = _(message='Attach tags to %(count)d document.')
    view_icon = icon_document_tag_multiple_attach

    def get_extra_context(self):
        context = {}

        if self.object_list.count() == 1:
            context.update(
                {
                    'object': self.object_list.first()
                }
            )

        return context

    def get_form_extra_kwargs(self):
        kwargs = {
            'help_text': _(message='Tags to be attached.'),
            'permission': permission_tag_attach,
            'queryset': Tag.objects.all(),
            'user': self.request.user
        }

        if self.object_list.count() == 1:
            kwargs.update(
                {
                    'queryset': Tag.objects.exclude(
                        pk__in=self.object_list.first().tags.all()
                    )
                }
            )

        return kwargs

    def get_post_action_redirect(self):
        if self.object_list.count() == 1:
            return reverse(
                kwargs={
                    'document_id': self.object_list.first().pk
                }, viewname='tags:document_tag_list'
            )
        else:
            return super().get_post_action_redirect()

    def object_action(self, form, instance):
        for tag in form.cleaned_data['tags']:
            AccessControlList.objects.check_access(
                obj=tag, permission=permission_tag_attach,
                user=self.request.user
            )

            tag.attach_to(document=instance, user=self.request.user)


class DocumentTagListView(ExternalObjectViewMixin, TagListView):
    external_object_permission = permission_tag_view
    external_object_pk_url_kwarg = 'document_id'
    external_object_queryset = Document.valid.all()
    tag_model = DocumentTag
    view_icon = icon_document_tag_list

    def get_extra_context(self):
        context = super().get_extra_context()
        context.update(
            {
                'hide_link': True,
                'no_results_main_link': link_document_tag_multiple_attach.resolve(
                    context=RequestContext(
                        dict_={'object': self.external_object},
                        request=self.request
                    )
                ),
                'no_results_title': _(
                    message='Document has no tags attached'
                ),
                'object': self.external_object,
                'title': _(
                    message='Tags for document: %s'
                ) % self.external_object
            }
        )
        return context

    def get_tag_queryset(self):
        return self.external_object.get_tags(
            permission=permission_tag_view, user=self.request.user
        )


class TagRemoveActionView(MultipleObjectFormActionView):
    form_class = TagMultipleSelectionForm
    object_permission = permission_tag_remove
    pk_url_kwarg = 'document_id'
    source_queryset = Document.valid.all()
    success_message_plural = _(
        message='Tags removed from %(count)d documents successfully.'
    )
    success_message_single = _(
        message='Tags removed from document "%(object)s" successfully.'
    )
    success_message_singular = _(
        message='Tags removed from %(count)d document successfully.'
    )
    title_plural = _(message='Remove tags from %(count)d documents.')
    title_single = _(message='Remove tags from document: %(object)s')
    title_singular = _(message='Remove tags from %(count)d document.')
    view_icon = icon_document_tag_multiple_remove

    def get_extra_context(self):
        context = {}

        if self.object_list.count() == 1:
            context.update(
                {
                    'object': self.object_list.first()
                }
            )

        return context

    def get_form_extra_kwargs(self):
        kwargs = {
            'help_text': _(message='Tags to be removed.'),
            'permission': permission_tag_remove,
            'queryset': Tag.objects.all(),
            'user': self.request.user
        }

        if self.object_list.count() == 1:
            kwargs.update(
                {
                    'queryset': self.object_list.first().tags.all()
                }
            )

        return kwargs

    def get_post_action_redirect(self):
        if self.object_list.count() == 1:
            return reverse(
                kwargs={
                    'document_id': self.object_list.first().pk
                }, viewname='tags:document_tag_list'
            )
        else:
            return super().get_post_action_redirect()

    def object_action(self, form, instance):
        for tag in form.cleaned_data['tags']:
            AccessControlList.objects.check_access(
                obj=tag, permission=permission_tag_remove,
                user=self.request.user
            )

            tag.remove_from(document=instance, user=self.request.user)



from mayan.apps.forms import forms

from .models import StoredPermission


class StoredPermissionDetailForm(forms.DetailForm):
    class Meta:
        fields = ('namespace', 'name')
        model = StoredPermission



from django.utils.translation import gettext_lazy as _

from mayan.apps.smart_settings.settings import setting_cluster

from .literals import (
    DEFAULT_DOCUMENT_PARSING_AUTO_PARSING,
    DEFAULT_DOCUMENT_PARSING_PDFTOTEXT_PATH
)

setting_namespace = setting_cluster.do_namespace_add(
    label=_(message='Document parsing'), name='document_parsing'
)

setting_auto_parsing = setting_namespace.do_setting_add(
    choices=('false', 'true'),
    default=DEFAULT_DOCUMENT_PARSING_AUTO_PARSING,
    global_name='DOCUMENT_PARSING_AUTO_PARSING', help_text=_(
        message='Set new document types to perform parsing automatically by default.'
    )
)
setting_pdftotext_path = setting_namespace.do_setting_add(
    default=DEFAULT_DOCUMENT_PARSING_PDFTOTEXT_PATH,
    global_name='DOCUMENT_PARSING_PDFTOTEXT_PATH', help_text=_(
        message='File path to poppler\'s pdftotext program used to extract text '
        'from PDF files.'
    ), is_path=True
)



import hashlib
import logging

from furl import furl
from graphviz import Digraph

from django.apps import apps
from django.core import serializers
from django.db import IntegrityError
from django.urls import reverse
from django.utils.functional import cached_property
from django.utils.translation import gettext_lazy as _

from mayan.apps.acls.models import AccessControlList
from mayan.apps.documents.models.document_models import Document
from mayan.apps.documents.permissions import permission_document_view
from mayan.apps.file_caching.models import CachePartitionFile

from ..events import event_workflow_template_edited
from ..literals import (
    ERROR_LOG_DOMAIN_NAME, GRAPHVIZ_RANKDIR, GRAPHVIZ_RANKSEP,
    GRAPHVIZ_SPLINES, STORAGE_NAME_WORKFLOW_CACHE
)

logger = logging.getLogger(name=__name__)


class WorkflowBusinessLogicMixin:
    @cached_property
    def cache(self):
        Cache = apps.get_model(app_label='file_caching', model_name='Cache')

        return Cache.objects.get(
            defined_storage_name=STORAGE_NAME_WORKFLOW_CACHE
        )

    @cached_property
    def cache_partition(self):
        partition, created = self.cache.partitions.get_or_create(
            name='{}'.format(self.pk)
        )
        return partition

    def do_diagram_generate(self):
        diagram = Digraph(
            name='finite_state_machine', graph_attr={
                'rankdir': GRAPHVIZ_RANKDIR, 'ranksep': GRAPHVIZ_RANKSEP,
                'splines': GRAPHVIZ_SPLINES
            }, format='png'
        )

        for state in self.states.order_by('completion', 'label'):
            state.do_diagram_generate(diagram=diagram)

        for transition in self.transitions.all():
            transition.do_diagram_generate(diagram=diagram)

        return diagram.pipe()

    def document_types_add(self, queryset, user):
        for model_instance in queryset.all():
            self.document_types.add(model_instance)
            event_workflow_template_edited.commit(
                action_object=model_instance, actor=user, target=self
            )

    def document_types_remove(self, queryset, user):
        for model_instance in queryset.all():
            self.document_types.remove(model_instance)
            event_workflow_template_edited.commit(
                action_object=model_instance, actor=user, target=self
            )
            self.instances.filter(
                document__document_type_id=model_instance.pk
            ).delete()

    def generate_image(
        self, maximum_layer_order=None, transformation_instance_list=None,
        user=None
    ):
        # `user` argument added for compatibility.
        cache_filename = '{}'.format(
            self.get_hash()
        )

        try:
            self.cache_partition.get_file(filename=cache_filename)
        except CachePartitionFile.DoesNotExist:
            logger.debug(
                'workflow cache file "%s" not found', cache_filename
            )

            image = self.do_diagram_generate()
            with self.cache_partition.create_file(filename=cache_filename) as file_object:
                file_object.write(image)
        else:
            logger.debug(
                'workflow cache file "%s" found', cache_filename
            )

        return cache_filename

    def get_api_image_url(self, *args, **kwargs):
        final_url = furl()
        final_url.args = kwargs
        final_url.path = reverse(
            kwargs={'workflow_template_id': self.pk},
            viewname='rest_api:workflow-template-image'
        )
        final_url.args['_hash'] = self.get_hash()

        return final_url.tostr()

    def get_document_types_not_in_workflow(self):
        DocumentType = apps.get_model(
            app_label='documents', model_name='DocumentType'
        )

        return DocumentType.objects.exclude(
            pk__in=self.document_types.all()
        )

    def get_hash(self):
        result = hashlib.sha256(
            string=serializers.serialize(
                format='json', queryset=(self,)
            ).encode()
        )
        for state in self.states.all():
            result.update(
                state.get_hash().encode()
            )

        for transition in self.transitions.all():
            result.update(
                transition.get_hash().encode()
            )

        return result.hexdigest()

    def get_state_final(self):
        try:
            return self.states.get(final=True)
        except self.states.model.DoesNotExist:
            return None
    get_state_final.short_description = _(message='Final state')

    def get_state_initial(self):
        try:
            return self.states.get(initial=True)
        except self.states.model.DoesNotExist:
            return None
    get_state_initial.short_description = _(message='Initial state')

    def launch_for(self, document, user=None):
        WorkflowInstance = apps.get_model(
            app_label='document_states',
            model_name='WorkflowInstance'
        )

        initial_state = self.get_state_initial()

        if initial_state:
            queryset = self.document_types.all()
            if queryset.filter(pk=document.document_type.pk).exists():
                try:
                    logger.debug(
                        'Launching workflow %s for document %s', self, document
                    )
                    workflow_instance = WorkflowInstance(
                        document=document, workflow=self
                    )
                    workflow_instance._event_actor = user
                    workflow_instance.save()

                    initial_state.do_active_set(
                        workflow_instance=workflow_instance
                    )
                    # TODO: Update once initial entry log patch is merged.
                    # Break pattern by passing `workflow_instance`
                    # until initial entry logs patch is merged.
                except IntegrityError:
                    logger.debug(
                        'Workflow %s already launched for document %s',
                        self, document
                    )
                else:
                    logger.debug(
                        'Workflow %s launched for document %s', self, document
                    )
                    return workflow_instance
            else:
                logger.error(
                    'This workflow is not valid for the document type of the '
                    'document.'
                )
        else:
            text_error = '''
            Cannot create a workflow instance. The workflow template `{}`
            does not have an initial state.
            '''.format(self)

            document.error_log.create(
                domain_name=ERROR_LOG_DOMAIN_NAME, text=text_error
            )


class WorkflowRuntimeProxyBusinessLogicMixin:
    def get_documents(self, permission=None, user=None):
        """
        Provide a queryset of the documents. The queryset is optionally
        filtered by access.
        """
        queryset = Document.valid.filter(workflows__workflow=self)

        if self.ignore_completed:
            queryset = queryset.exclude(workflows__state_active__final=True)

        if permission and user:
            queryset = AccessControlList.objects.restrict_queryset(
                permission=permission, queryset=queryset,
                user=user
            )

        return queryset

    def get_document_count(self, user):
        """
        Return the numeric count of documents executing this workflow.
        The count is filtered by access.
        """
        return self.get_documents(
            permission=permission_document_view, user=user
        ).count()

    def get_states(self):
        WorkflowStateRuntimeProxy = apps.get_model(
            app_label='document_states',
            model_name='WorkflowStateRuntimeProxy'
        )

        queryset = WorkflowStateRuntimeProxy.objects.filter(workflow=self)

        if self.ignore_completed:
            queryset = queryset.exclude(final=True)

        return queryset



from django.template import Library

import mayan
from mayan.apps.dependencies.versions import Version
from mayan.apps.platforms.utils import yaml_dump
from mayan.settings.literals import LINUX_PACKAGES_DEBIAN_PUSH

register = Library()


@register.simple_tag(name='platforms_gitlab_ci_cache_before_script')
def tag_platforms_gitlab_ci_cache_before_script(indent, apk=False, apt=False):
    data = []

    if apk:
        data.extend(
            [
                'mkdir --parents ${APK_CACHE_DIR}',
                'apk update'
            ]
        )

    if apt:
        data.extend(
            [
                'export APT_STATE_LISTS=${APT_CACHE_DIR}/lists && export APT_CACHE_ARCHIVES=${APT_CACHE_DIR}/archives',
                'mkdir --parents "${APT_STATE_LISTS}/partial" && mkdir --parents "${APT_CACHE_ARCHIVES}/partial"',
                'printf "dir::state::lists    ${APT_STATE_LISTS};\\ndir::cache::archives    ${APT_CACHE_ARCHIVES};\\n" > /etc/apt/apt.conf.d/99gitlab-ci-cache',
                'if [ "${APT_PROXY}" ]; then echo "Acquire::http { Proxy \\"http://${APT_PROXY}\\"; };" > /etc/apt/apt.conf.d/01proxy; fi',
                'apt-get update'
            ]
        )

    return yaml_dump(data=data, indent=indent)


@register.simple_tag(name='platforms_gitlab_ci_cache_paths')
def tag_platforms_gitlab_ci_cache_paths(
    indent, apk=False, apt=False, pip=False
):
    cache_list = []

    version_base = Version(version_string=mayan.__version__)
    version_upstream = Version(
        version_string=version_base.as_upstream()
    )
    version_final = version_upstream.as_minor()

    if apk:
        cache_list.append(
            {
                'key': 'apk-cache-{}'.format(version_final),
                'paths': ['${APK_CACHE_DIR}']
            }
        )

    if apt:
        cache_list.append(
            {
                'key': 'apt-cache-{}'.format(version_final),
                'paths': ['${APT_CACHE_DIR}']
            }
        )

    if pip:
        cache_list.append(
            {
                'key': 'pip-cache-{}'.format(version_final),
                'paths': ['${PIP_CACHE_DIR}']
            }
        )

    return yaml_dump(data=cache_list, indent=indent)


@register.simple_tag(name='platforms_gitlab_ci_cache_variables')
def tag_platforms_gitlab_ci_cache_variables(
    indent, apk=False, apt=False, pip=False
):
    variables = {}

    if apk:
        variables['APK_CACHE_DIR'] = '${CI_PROJECT_DIR}/.cache/apk'

    if apt:
        variables['APT_CACHE_DIR'] = '${CI_PROJECT_DIR}/.cache/apt'

    if pip:
        variables['PIP_CACHE_DIR'] = '${CI_PROJECT_DIR}/.cache/pip'

    return yaml_dump(data=variables, indent=indent)


@register.simple_tag(name='platforms_gitlab_ci_config_env_before_script')
def tag_platforms_gitlab_ci_config_env_before_script(indent):
    data = [
        r'set -a && sed -E "s/=(.*)/\=\"\1\"/g" config.env > /tmp/config.env && . /tmp/config.env && rm /tmp/config.env && set +a'
    ]

    return yaml_dump(data=data, indent=indent)


@register.simple_tag(name='platforms_gitlab_ci_ssh_before_script')
def tag_platforms_gitlab_ci_ssh_before_script(indent, hostname, private_key):
    data = [
        'mkdir --parents ~/.ssh',
        'chmod 700 ~/.ssh',
        'echo "{}" > ~/.ssh/known_hosts'.format(hostname),
        'chmod 644 ~/.ssh/known_hosts',
        '\'which ssh-agent || ( apt-get update --yes && apt-get install --yes --no-install-recommends {debian_packages} )\''.format(debian_packages=LINUX_PACKAGES_DEBIAN_PUSH),
        'eval $(ssh-agent -s)',
        'echo "{}" | tr -d \'\\r\' | ssh-add - > /dev/null'.format(private_key)
    ]

    return yaml_dump(data=data, indent=indent)



from django.utils.translation import gettext_lazy as _

from mayan.apps.acls.classes import ModelPermission
from mayan.apps.acls.permissions import (
    permission_acl_edit, permission_acl_view
)
from mayan.apps.app_manager.apps import MayanAppConfig
from mayan.apps.common.menus import (
    menu_list_facet, menu_object, menu_related, menu_secondary, menu_setup
)
from mayan.apps.events.classes import EventModelRegistry, ModelEventType
from mayan.apps.navigation.source_columns import SourceColumn

from .classes import KeyStub
from .events import event_key_downloaded
from .links import (
    link_key_delete, link_key_detail, link_key_download, link_key_query,
    link_key_receive, link_key_setup, link_key_upload, link_private_key_list,
    link_public_key_list
)
from .permissions import (
    permission_key_delete, permission_key_download, permission_key_sign,
    permission_key_view
)


class DjangoGPGApp(MayanAppConfig):
    app_namespace = 'django_gpg'
    app_url = 'keys'
    has_rest_api = True
    has_tests = True
    name = 'mayan.apps.django_gpg'
    verbose_name = _(message='Django GPG')

    def ready(self):
        super().ready()

        Key = self.get_model(model_name='Key')

        EventModelRegistry.register(model=Key)
        ModelEventType.register(
            model=Key, event_types=(
                event_key_downloaded,
            )
        )
        ModelPermission.register(
            model=Key, permissions=(
                permission_acl_edit, permission_acl_view,
                permission_key_delete, permission_key_download,
                permission_key_sign, permission_key_view
            )
        )

        SourceColumn(
            attribute='key_id', is_identifier=True, label=_(message='Key ID'),
            source=Key
        )
        SourceColumn(attribute='user_id', include_label=True, source=Key)

        SourceColumn(
            attribute='key_id', include_label=True, label=_(message='Key ID'),
            source=KeyStub
        )
        SourceColumn(
            attribute='key_type', include_label=True, label=_(message='Type'),
            source=KeyStub
        )
        SourceColumn(
            attribute='date', include_label=True, label=_(message='Creation date'),
            source=KeyStub
        )
        SourceColumn(
            func=lambda context: context['object'].expires or _(
                message='No expiration'
            ), include_label=True, label=_(message='Expiration date'),
            source=KeyStub
        )
        SourceColumn(
            attribute='length', include_label=True, label=_(message='Length'),
            source=KeyStub
        )
        SourceColumn(
            func=lambda context: ', '.join(context['object'].user_id),
            include_label=True, label=_(message='User ID'), source=KeyStub
        )

        menu_list_facet.bind_links(
            links=(link_key_detail,), sources=(Key,)
        )
        menu_object.bind_links(
            links=(link_key_receive,), sources=(KeyStub,)
        )

        menu_object.bind_links(
            links=(link_key_delete, link_key_download),
            sources=(Key,)
        )
        menu_related.bind_links(
            links=(link_private_key_list, link_public_key_list),
            sources=(
                'django_gpg:key_public_list', 'django_gpg:key_private_list',
                'django_gpg:key_query', 'django_gpg:key_query_results',
                'django_gpg:key_upload', Key, KeyStub
            )
        )
        menu_secondary.bind_links(
            links=(link_key_query, link_key_upload),
            sources=(
                'django_gpg:key_public_list', 'django_gpg:key_private_list',
                'django_gpg:key_query', 'django_gpg:key_query_results',
                'django_gpg:key_upload', Key, KeyStub
            )
        )
        menu_setup.bind_links(
            links=(link_key_setup,)
        )



from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('acls', '0004_auto_20210130_0322')
    ]

    operations = [
        migrations.AlterField(
            model_name='accesscontrollist',
            name='content_type',
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name='object_content_type',
                to='contenttypes.contenttype', verbose_name='Content type'
            )
        ),
        migrations.AlterField(
            model_name='accesscontrollist',
            name='object_id',
            field=models.PositiveIntegerField(verbose_name='Object ID')
        )
    ]



from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('metadata', '0018_alter_documentmetadata_value')
    ]

    operations = [
        migrations.AlterField(
            field=models.TextField(
                blank=True, help_text='The actual value stored '
                'in the metadata type field for the document.',
                null=True, verbose_name='Value'
            ), model_name='documentmetadata', name='value'
        )
    ]



from django.utils.functional import cached_property
from django.utils.module_loading import import_string
from django.utils.translation import gettext_lazy as _

from mayan.apps.common.serialization import yaml_load


class DocumentFileDriverEntryBusinessLogicMixin:
    def get_attribute_count(self):
        return self.entries.count()
    get_attribute_count.short_description = _(message='Attribute count')


class DocumentTypeDriverConfigurationBusinessLogicMixin:
    def get_arguments(self):
        driver_class = self.stored_driver.driver_class

        argument_values_from_settings = driver_class.get_argument_values_from_settings()

        stored_arguments = yaml_load(stream=self.arguments or '{}')

        for argument_name in self.stored_driver.driver_class.get_argument_name_list():
            value = stored_arguments.get(argument_name)

            if value:
                argument_values_from_settings[argument_name] = value

        return argument_values_from_settings

    def get_arguments_display(self):
        try:
            arguments = self.get_arguments()
        except Exception as exception:
            arguments = {
                'error': _(
                    message='Badly formatted arguments YAML; %s'
                ) % exception
            }

        result = []
        for key, value in arguments.items():
            result.append(
                '{}: {}'.format(key, value)
            )

        return ', '.join(result)

    get_arguments_display.short_description = _(message='Arguments')


class FileMetadataEntryBusinessLogicMixin:
    def get_full_path(self):
        return '{}__{}'.format(
            self.document_file_driver_entry.driver.internal_name,
            self.internal_name
        )

    get_full_path.short_description = _(message='Full path')
    get_full_path.help_text = _(
        message='Path used to access the value of the file metadata entry.'
    )


class StoredDriverBusinessLogicMixin:
    @cached_property
    def driver_class(self):
        return import_string(dotted_path=self.driver_path)

    @cached_property
    def driver_label(self):
        return self.driver_class.label



from django.core.exceptions import ValidationError
from django.db import models
from django.db.models import Value
from django.db.models.functions import Cast, Replace

from ...exceptions import DynamicSearchValueTransformationError
from ...search_backends import SearchBackend
from ...search_fields import SearchFieldVirtualAllFields
from ...search_models import SearchModel

from .literals import DJANGO_TO_DJANGO_FIELD_MAP


class DjangoSearchBackend(SearchBackend):
    field_type_mapping = DJANGO_TO_DJANGO_FIELD_MAP

    def _do_search_model_filter(self, filter_kwargs, search_field):
        if search_field.field_class == models.UUIDField:
            # Remove hyphens when searching UUID fields.
            replace_function = Replace(
                expression=Cast(
                    expression=search_field.field_name,
                    output_field=models.CharField()
                ), text=Value('-'), replacement=Value(''),
                output_field=models.CharField()
            )
        else:
            replace_function = Replace(
                expression=Cast(
                    expression=search_field.field_name,
                    output_field=models.CharField()
                ), text=Value('-'), replacement=Value('_'),
                output_field=models.CharField()
            )

        queryset = search_field.search_model.get_queryset().annotate(
            **{
                '{}_clean'.format(search_field.field_name): replace_function
            }
        )

        try:
            queryset = queryset.filter(filter_kwargs)
            values = queryset.values_list('pk', flat=True)

            values_unique = tuple(
                set(values)
            )

            for entry in values_unique:
                yield entry
        except ValidationError:
            return ()

    def _get_status(self):
        result = []

        for search_model in SearchModel.all():
            queryset = search_model.get_queryset()

            result.append(
                '{}: {}'.format(
                    search_model.label, queryset.count()
                )
            )

        return '\n'.join(result)

    def _search(
        self, search_field, query_type, value, is_quoted_value=False,
        is_raw_value=False
    ):
        self.do_query_type_verify(
            query_type=query_type, search_field=search_field
        )

        if isinstance(search_field, SearchFieldVirtualAllFields):
            result = set()

            for search_field in search_field.field_composition:
                try:
                    search_field_query = query_type.resolve_for_backend(
                        is_quoted_value=is_quoted_value,
                        is_raw_value=is_raw_value, search_backend=self,
                        search_field=search_field, value=value
                    )
                except DynamicSearchValueTransformationError:
                    """Skip the search field."""
                else:
                    if search_field_query is not None:
                        field_id_list = self._do_search_model_filter(
                            filter_kwargs=search_field_query,
                            search_field=search_field
                        )

                        result.update(field_id_list)

            return result
        else:
            try:
                filter_kwargs = query_type.resolve_for_backend(
                    is_quoted_value=is_quoted_value,
                    is_raw_value=is_raw_value, search_backend=self,
                    search_field=search_field, value=value
                )
            except DynamicSearchValueTransformationError:
                return ()
            else:
                if filter_kwargs is None:
                    return ()
                else:
                    return self._do_search_model_filter(
                        filter_kwargs=filter_kwargs, search_field=search_field
                    )



import logging

from django.contrib import messages
from django.http import HttpResponseRedirect, JsonResponse
from django.urls import reverse
from django.utils.translation import gettext_lazy as _

from mayan.apps.documents.models.document_models import Document
from mayan.apps.documents.models.document_type_models import DocumentType
from mayan.apps.documents.permissions import permission_document_create
from mayan.apps.views.utils import request_is_ajax
from mayan.apps.views.view_mixins import ExternalObjectViewMixin

from ..forms import NewDocumentForm
from ..icons import icon_document_upload_wizard

from .base import UploadBaseView

logger = logging.getLogger(name=__name__)


class DocumentUploadView(ExternalObjectViewMixin, UploadBaseView):
    document_form = NewDocumentForm
    external_object_class = DocumentType
    external_object_permission = permission_document_create
    object_permission = permission_document_create
    source_link_view_name = 'sources:document_upload'
    view_icon = icon_document_upload_wizard
    view_source_action = 'document_upload'

    def forms_valid(self, forms):
        action = self.source.get_action(name='document_upload')

        interface_load_kwargs = {
            'document_type': self.external_object, 'forms': forms,
            'request': self.request, 'view': self
        }

        try:
            Document.execute_pre_create_hooks(
                kwargs={
                    'document_type': self.external_object,
                    'file_object': None,
                    'user': self.request.user
                }
            )
            action.execute(
                interface_name='View',
                interface_load_kwargs=interface_load_kwargs
            )
        except Exception as exception:
            message = _(
                message='Error processing source document upload; '
                '%(exception)s'
            ) % {
                'exception': exception,
            }
            logger.critical(msg=message, exc_info=True)
            if request_is_ajax(request=self.request):
                return JsonResponse(
                    data={
                        'error': str(message)
                    }, status=500
                )
            else:
                messages.error(
                    message=message.replace('\n', ' '),
                    request=self.request
                )
                raise type(exception)(message) from exception
        else:
            messages.success(
                message=_(
                    message='New document queued for upload and will be '
                    'available shortly.'
                ), request=self.request
            )

            return HttpResponseRedirect(
                redirect_to='{}?{}'.format(
                    reverse(
                        kwargs=self.request.resolver_match.kwargs,
                        viewname=self.request.resolver_match.view_name
                    ), self.request.META['QUERY_STRING']
                ),
            )

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context.update(
            {
                'document_type': self.external_object,
                'title': _(
                    message='Upload a document of type "%(document_type)s" '
                    'from source: %(source)s'
                ) % {
                    'document_type': self.external_object,
                    'source': self.source.label
                }
            }
        )

        return context

    def get_form_extra_kwargs__document_form(self):
        return {'document_type': self.external_object}

    def get_form_extra_kwargs__source_form(self):
        return {'source': self.source}

    def get_pk_url_kwargs(self):
        return {
            'pk': self.request.GET.get(
                'document_type_id', self.request.POST.get('document_type_id')
            )
        }



import base64
from collections import namedtuple
import io
import json

from furl import furl

from django.core.handlers.wsgi import WSGIRequest
from django.http.request import QueryDict
from django.template import Variable, VariableDoesNotExist
from django.test.client import MULTIPART_CONTENT
from django.urls import resolve
from django.urls.exceptions import Resolver404
from django.utils.encoding import force_bytes, force_str

from mayan.apps.organizations.settings import (
    setting_organization_url_base_path
)
from mayan.apps.templating.template_backends import Template

from .literals import API_VERSION


class BatchResponse:
    def __init__(self, content, name, status_code, data, headers):
        self.content = content
        self.name = name
        self.status_code = status_code
        self.data = data
        self.headers = headers


class NestableLazyIterator:
    def __init__(
        self, iterable_string, context, context_list_index,
        parent_iterator=None
    ):
        self.iterable_string = iterable_string
        self.context = context
        self.context_list_index = context_list_index
        self.parent_iterator = parent_iterator
        self.items = None
        self.index = 0

    def __iter__(self):
        return self

    def __next__(self):
        # Setup the initial values on the initial access.
        if not self.items:
            if self.parent_iterator:
                next(self.parent_iterator)

            self.update_iterable_object()

        if self.index == len(self.items):
            self.index = 0

            if self.parent_iterator:
                next(self.parent_iterator)
            else:
                raise StopIteration

            self.update_iterable_object()

        value = self.items[self.index]
        self.context['iterables'][self.context_list_index] = value
        self.index += 1

        return value

    def update_iterable_object(self):
        self.items = Variable(
            var=self.iterable_string
        ).resolve(context=self.context)


RenderedContent = namedtuple(
    typename='RenderedContent', field_names=(
        'body', 'include', 'method', 'name', 'url'
    )
)


class BatchRequest:
    def __init__(
        self, collection, name, url, body=None, group_name=None,
        include='true', iterables=None, method='GET'
    ):
        self.collection = collection
        self.body = body or {}
        self.group_name = group_name
        self.include = include
        self.iterables = iterables
        self.method = method
        self.name = name
        self.url = url

    def execute(self):
        if self.iterables:
            # Initialize the iterables list to allow using any index.
            self.collection.context['iterables'] = [None] * len(
                self.iterables
            )

            iterator = None
            for iterable_index, iterable in enumerate(self.iterables):
                iterator = NestableLazyIterator(
                    context=self.collection.context,
                    context_list_index=iterable_index,
                    iterable_string=iterable, parent_iterator=iterator
                )

            while True:
                try:
                    next(iterator)
                except StopIteration:
                    break
                except VariableDoesNotExist as exception:
                    self.collection.responses[self.name] = {
                        'data': {
                            'error': str(exception)
                        },
                        'include': 'true',
                        'is_response': True
                    }
                    return
                else:
                    rendered_content = self.render_content()

                    BatchRequest(
                        collection=self.collection,
                        body=rendered_content.body,
                        group_name=self.group_name,
                        include=rendered_content.include,
                        method=rendered_content.method,
                        name=rendered_content.name,
                        url=rendered_content.url
                    ).execute()
        else:
            rendered_content = self.render_content()

            url_parts = furl(rendered_content.url)

            try:
                resolver_match = resolve(path=url_parts.pathstr)
            except Resolver404 as exception:
                self.collection.responses[rendered_content.name] = {
                    'data': {
                        'error': '"{}" not found'.format(
                            exception.args[0]['path']
                        )
                    },
                    'include': 'true',
                    'is_response': True,
                    'status_code': 404
                }
                return
            else:
                environ = getattr(
                    self.collection.view_request, 'environ', {}
                ).copy()

                environ['REQUEST_METHOD'] = rendered_content.method
                environ['PATH_INFO'] = self.url
                environ['QUERY_STRING'] = url_parts.querystr

                post_query_dict = QueryDict(mutable=True)
                post_query_dict.update(rendered_content.body)
                json_body = json.dumps(post_query_dict)
                request_data = json_body.encode('utf-8')
                environ['wsgi.input'] = io.BytesIO(
                    initial_bytes=request_data
                )
                environ['CONTENT_LENGTH'] = str(
                    len(request_data)
                )

                if rendered_content.method == 'POST':
                    environ['CONTENT_TYPE'] = MULTIPART_CONTENT
                else:
                    environ['CONTENT_TYPE'] = 'application/octet-stream'

                request = WSGIRequest(environ=environ)
                request.LANGUAGE_CODE = getattr(
                    self.collection.view_request, 'LANGUAGE_CODE', None
                )
                request.POST = post_query_dict
                request._read_started = True
                request.auth = getattr(
                    self.collection.view_request, 'auth', None
                )
                request.csrf_processing_done = True
                request.session = getattr(
                    self.collection.view_request, 'session', None
                )
                request.user = getattr(
                    self.collection.view_request, 'user', None
                )

                response = resolver_match.func(
                    request=request, **resolver_match.kwargs
                )

                result = {
                    'headers': {
                        key: value for key, value in response.items()
                    },
                    'include': rendered_content.include,
                    'is_response': True,
                    'status_code': response.status_code
                }

                try:
                    result['data'] = response.data
                except AttributeError:
                    try:
                        content = response.content
                    except AttributeError:
                        content = list(response.streaming_content)[0]

                    result['content'] = force_str(
                        s=base64.b64encode(
                            s=force_bytes(s=content)
                        )
                    )

                self.collection.context[rendered_content.name] = result
                self.collection.responses[rendered_content.name] = result

            if self.group_name:
                self.collection.context.setdefault(
                    'groups', {}
                )
                self.collection.context['groups'].setdefault(
                    self.group_name, []
                )
                self.collection.context['groups'][self.group_name].append(
                    result
                )

    def render_content(self):
        rendered_body = {}
        for key, value in self.body.items():
            rendered_key = Template(template_string=key).render(
                context=self.collection.context
            )
            rendered_value = Template(template_string=value).render(
                context=self.collection.context
            )
            rendered_body[rendered_key] = rendered_value

        rendered_include = Template(template_string=self.include).render(
            context=self.collection.context
        )
        rendered_method = Template(template_string=self.method).render(
            context=self.collection.context
        )
        rendered_name = Template(template_string=self.name).render(
            context=self.collection.context
        )
        rendered_url = Template(template_string=self.url).render(
            context=self.collection.context
        )

        return RenderedContent(
            body=rendered_body, include=rendered_include,
            method=rendered_method, name=rendered_name, url=rendered_url
        )


class BatchRequestCollection:
    def __init__(self, request_list=None):
        self.requests = []

        for request_index, request_dict in enumerate(request_list):
            request_dict.update(
                {'collection': self}
            )
            try:
                self.requests.append(
                    BatchRequest(**request_dict)
                )
            except Exception as exception:
                raise ValueError(
                    'Error instantiating request #{}; {}'.format(
                        request_index, exception
                    )
                ) from exception

    def execute(self, view_request):
        self.context = {'view_request': view_request}
        self.responses = {}
        self.view_request = view_request

        for request in self.requests:
            request.execute()

        # Convert responses in context into response class instances.
        result = []
        for key, value in self.responses.items():
            if json.loads(s=value.get('include', 'true')):
                result.append(
                    BatchResponse(
                        content=value.get('content', ''),
                        data=value.get(
                            'data', {}
                        ),
                        headers=value.get(
                            'headers', {}
                        ),
                        name=key,
                        status_code=value.get('status_code', 0),
                    )
                )

        return result


class Endpoint:
    def __init__(self, label, viewname=None, kwargs=None):
        self.label = label
        self.kwargs = kwargs

        if viewname:
            self.viewname = viewname
        else:
            installation_base_url = setting_organization_url_base_path.value
            if installation_base_url:
                installation_base_url = '/{}'.format(installation_base_url)
            else:
                installation_base_url = ''

            self.url = '{}/api/v{}/{}/'.format(
                installation_base_url, API_VERSION, self.label
            )

            try:
                self.viewname = resolve(path=self.url).view_name
            except Resolver404:
                self.viewname = None



from django.utils.translation import gettext_lazy as _

from mayan.apps.storage.classes import DefinedStorage

from .literals import STORAGE_NAME_SIGNATURE_CAPTURES_CACHE
from .settings import (
    setting_signature_capture_cache_storage_backend,
    setting_signature_capture_cache_storage_backend_arguments
)

storage_signature_captures_cache = DefinedStorage(
    dotted_path=setting_signature_capture_cache_storage_backend.value,
    error_message=_(
        message='Unable to initialize the converter signature capture cache'
        'storage. Check the settings {} and {} for formatting '
        'errors.'.format(
            setting_signature_capture_cache_storage_backend.global_name,
            setting_signature_capture_cache_storage_backend_arguments.global_name
        )
    ), label=_(message='Signature captures cache'),
    name=STORAGE_NAME_SIGNATURE_CAPTURES_CACHE,
    kwargs=setting_signature_capture_cache_storage_backend_arguments.value
)



from django.utils.translation import gettext_lazy as _

from mayan.apps.documents.search import search_model_document
from mayan.apps.dynamic_search.search_models import SearchModel

from .permissions import permission_cabinet_view

# Cabinet

search_model_cabinet = SearchModel(
    app_label='cabinets', model_name='CabinetSearchResult',
    permission=permission_cabinet_view,
    serializer_path='mayan.apps.cabinets.serializers.CabinetSerializer'
)
search_model_cabinet.add_proxy_model(
    app_label='cabinets', model_name='Cabinet'
)

search_model_cabinet.add_model_field(field='id')
search_model_cabinet.add_model_field(field='label')

# Cabinet documents

search_model_cabinet.add_model_field(
    field='documents__document_type__label', label=_(message='Document type')
)
search_model_cabinet.add_model_field(
    field='documents__label', label=_(message='Document label')
)
search_model_cabinet.add_model_field(
    field='documents__description', label=_(message='Document description')
)
search_model_cabinet.add_model_field(
    field='documents__uuid', label=_(message='Document UUID')
)

# Cabinet documents files

search_model_cabinet.add_model_field(
    field='documents__files__checksum', label=_(message='Document file checksum')
)
search_model_cabinet.add_model_field(
    field='documents__files__mimetype', label=_(message='Document file MIME type')
)

# Document

search_model_document.add_model_field(
    field='cabinets__id',
    help_text=_(message='The database ID of the cabinet.'),
    label=_(message='Cabinet ID')
)
search_model_document.add_model_field(
    field='cabinets__label', label=_(message='Cabinets')
)



#!/usr/bin/env python

import os
import sys

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

import mayan

PACKAGE_NAME = 'mayan-edms'
PACKAGE_DIR = 'mayan'

if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    sys.exit()


def fullsplit(path, result=None):
    """
    Split a pathname into components (the opposite of os.path.join) in a
    platform-neutral way.
    """
    if result is None:
        result = []
    head, tail = os.path.split(path)
    if head == '':
        return [tail] + result
    if head == path:
        return result
    return fullsplit(head, [tail] + result)


def find_packages(directory):
    # Compile the list of packages available, because distutils doesn't have
    # an easy way to do this.
    packages, data_files = [], []
    root_dir = os.path.dirname(__file__)
    if root_dir != '':
        os.chdir(root_dir)

    for dirpath, dirnames, filenames in os.walk(directory):
        if not dirpath.startswith('mayan/media'):
            # Ignore dirnames that start with '.'
            if os.path.basename(dirpath).startswith('.'):
                continue
            if '__init__.py' in filenames:
                packages.append(
                    '.'.join(
                        fullsplit(dirpath)
                    )
                )
            elif filenames:
                data_files.append(
                    [
                        dirpath, [
                            os.path.join(dirpath, filename) for filename in filenames
                        ]
                    ]
                )

    return packages


install_requires = """
django==4.2.24
CairoSVG==2.8.2
Markdown==3.9
Pillow==11.3.0
PyYAML==6.0.2
Whoosh==2.7.4
boto3==1.40.24
celery==5.5.3
dateparser==1.2.2
django-activity-stream==2.0.0
django-auth-ldap==5.1.0
django-celery-beat==2.8.1
django-cors-headers==4.7.0
django-formtools==2.3
django-model-utils==5.0.0
django-mptt==0.18.0
django-qsstats-magic==1.1.0
django-solo==2.4.0
django-storages==1.14.6
django-stronghold==0.4.0
django-widget-tweaks==1.5.0
djangorestframework==3.16.1
djangorestframework-recursive==0.1.2
drf-yasg==1.21.10
elasticsearch==9.1.0
extract-msg==0.55.0
flex==6.14.1
furl==2.1.4
fusepy==3.0.1
gevent==25.8.2
google-cloud-storage==3.3.1
graphviz==0.21
greenlet==3.2.4
gunicorn==23.0.0
jsonschema==4.25.1
mozilla-django-oidc==4.0.1
nh3==0.3.0
node-semver==0.9.0
ollama==0.5.3
openai==1.107.2
pycountry==24.6.1
pycryptodome==3.23.0
pyotp==2.9.0
pypdf==6.0.0
python-dateutil==2.9.0.post0
python_gnupg==0.5.5
pytz==2025.2
qrcode==8.2
requests==2.32.5
sentry-sdk==2.37.0
sh==2.2.2
swagger-spec-validator==3.0.4
whitenoise==6.9.0
""".split()

with open(file='README.rst') as file_object:
    readme = file_object.read()

setup(
    author='Roberto Rosario',
    author_email='roberto.rosario@mayan-edms.com',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Education',
        'Intended Audience :: Developers',
        'Intended Audience :: Information Technology',
        'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',
        'Natural Language :: English',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: WSGI :: Application',
        'Topic :: Communications :: File Sharing'
    ],
    description=mayan.__description__,
    include_package_data=True,
    install_requires=install_requires,
    license='"GPL-2.0-only',
    long_description=readme,
    name=PACKAGE_NAME,
    packages=find_packages(PACKAGE_DIR),
    platforms=['any'],
    project_urls={
        'Documentation': 'https://docs.mayan-edms.com/',
        'Forum': 'https://forum.mayan-edms.com/',
        'Changelog': 'https://gitlab.com/mayan-edms/mayan-edms/-/blob/master/HISTORY.rst',
        'Bug Tracker': 'https://gitlab.com/mayan-edms/mayan-edms/-/issues',
        'Source Code': 'https://gitlab.com/mayan-edms/mayan-edms',
        'Support': 'https://www.mayan-edms.com/support/'
    },
    python_requires='>=3.9',
    scripts=['mayan/bin/mayan-edms.py'],
    url=mayan.__website__,
    version=mayan.__version__,
    zip_safe=False
)



from django.apps import apps

from .settings import (
    setting_user_language_default, setting_user_timezone_default
)


def handler_user_locale_profile_create(sender, instance, created, **kwargs):
    UserLocaleProfile = apps.get_model(
        app_label='locales', model_name='UserLocaleProfile'
    )

    if created:
        UserLocaleProfile.objects.create(
            language=setting_user_language_default.value,
            timezone=setting_user_timezone_default.value, user=instance
        )



from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('logging', '0006_alter_errorlogpartition_unique_together')
    ]

    operations = [
        migrations.AddField(
            model_name='errorlogpartitionentry', name='domain_name',
            field=models.CharField(
                db_index=True, default='logging', max_length=128,
                verbose_name='Domain name'
            )
        )
    ]



from django.utils.translation import gettext_lazy as _

from mayan.apps.permissions.classes import PermissionNamespace

namespace = PermissionNamespace(
    label=_(message='Tags'), name='tags'
)

permission_tag_attach = namespace.add_permission(
    label=_(message='Attach tags to documents'), name='tag_attach'
)
permission_tag_create = namespace.add_permission(
    label=_(message='Create new tags'), name='tag_create'
)
permission_tag_delete = namespace.add_permission(
    label=_(message='Delete tags'), name='tag_delete'
)
permission_tag_view = namespace.add_permission(
    label=_(message='View tags'), name='tag_view'
)
permission_tag_edit = namespace.add_permission(
    label=_(message='Edit tags'), name='tag_edit'
)
permission_tag_remove = namespace.add_permission(
    label=_(message='Remove tags from documents'), name='tag_remove'
)



from django.db import models
from django.urls import reverse
from django.utils.translation import gettext_lazy as _

from mayan.apps.databases.model_mixins import ExtraDataModelMixin
from mayan.apps.documents.models.document_type_models import DocumentType
from mayan.apps.events.decorators import method_event
from mayan.apps.events.event_managers import EventManagerSave

from .events import event_web_link_created, event_web_link_edited
from .managers import WebLinkManager
from .model_mixins import (
    ResolvedWebLinkBusinessLogicMixin, WebLinkBusinessLogicMixin
)


class WebLink(ExtraDataModelMixin, WebLinkBusinessLogicMixin, models.Model):
    """
    This model stores the basic fields for a web link. Web links allow
    generating links from documents to external resources.
    """
    _ordering_fields = ('enabled', 'label')

    label = models.CharField(
        db_index=True, help_text=_(
            message='A short text describing the web link.'
        ), max_length=96, unique=True, verbose_name=_(message='Label')
    )
    template = models.TextField(
        help_text=_(
            message='Template that will be used to craft the final URL of the '
            'web link.'
        ), verbose_name=_(message='Template')
    )
    enabled = models.BooleanField(
        default=True, verbose_name=_(message='Enabled')
    )
    document_types = models.ManyToManyField(
        related_name='web_links', to=DocumentType,
        verbose_name=_(message='Document types')
    )

    class Meta:
        ordering = ('label',)
        verbose_name = _(message='Web link')
        verbose_name_plural = _(message='Web links')

    def __str__(self):
        return self.label

    def get_absolute_url(self):
        return reverse(
            kwargs={'web_link_id': self.pk},
            viewname='web_links:web_link_edit'
        )

    @method_event(
        event_manager_class=EventManagerSave,
        created={
            'event': event_web_link_created,
            'target': 'self'
        },
        edited={
            'event': event_web_link_edited,
            'target': 'self'
        }
    )
    def save(self, *args, **kwargs):
        return super().save(*args, **kwargs)


class ResolvedWebLink(ResolvedWebLinkBusinessLogicMixin, WebLink):
    """
    Proxy model to represent an already resolved web link. Used for easier
    columns registration.
    """
    objects = WebLinkManager()

    class Meta:
        proxy = True



from django.utils.translation import gettext_lazy as _

from mayan.apps.app_manager.apps import MayanAppConfig


class AppearanceBootstrapApp(MayanAppConfig):
    app_namespace = 'appearance_bootstrap'
    app_url = 'appearance_bootstrap'
    has_javascript_translations = True
    has_static_media = True
    name = 'mayan.apps.appearance_bootstrap'
    static_media_ignore_patterns = (
        'appearance_bootstrap/node_modules/bootswatch/docs/*',
    )
    verbose_name = _(message='Appearance (Bootstrap)')



from django.utils.translation import gettext_lazy as _

from mayan.apps.acls.models import AccessControlList
from mayan.apps.documents.models.document_models import Document
from mayan.apps.documents.permissions import permission_document_view


class CabinetBusinessLogicMixin:
    def _get_documents(self):
        return Document.valid.filter(
            pk__in=self.documents.values('pk')
        )

    def document_add(self, document, user):
        return self._document_add(document=document, user=user)

    def document_remove(self, document, user):
        return self._document_remove(document=document, user=user)

    def get_descendants_document_count(self, user):
        queryset_documents = Document.valid.filter(
            cabinets__in=self.get_descendants(
                include_self=True
            )
        ).distinct()

        queryset_documents_filtered = AccessControlList.objects.restrict_queryset(
            permission=permission_document_view,
            queryset=queryset_documents, user=user
        )

        return queryset_documents_filtered.count()

    def get_document_count(self, user):
        """
        Return numeric count of the total documents in a cabinet. The count
        is filtered by access.
        """
        return self.get_documents(
            permission=permission_document_view, user=user
        ).count()

    def get_documents(self, permission, user):
        """
        Provide a queryset of the documents in a cabinet. The queryset is
        filtered by access.
        """
        return AccessControlList.objects.restrict_queryset(
            permission=permission, queryset=self._get_documents(),
            user=user
        )

    def get_full_path(self):
        """
        Returns a string that represents the path to the cabinet. The
        path string starts from the root cabinet.
        """
        result = []
        for node in self.get_ancestors(include_self=True):
            result.append(node.label)

        return ' / '.join(result)
    get_full_path.help_text = _(
        message='The path to the cabinet including all ancestors.'
    )
    get_full_path.short_description = _(message='Full path')



import json

from django.db import models

from mayan.apps.databases.manager_mixins import ManagerMinixCreateBulk

from .literals import DOCUMENT_FILE_SOURCE_METADATA_BATCH_SIZE


class DocumentFileSourceMetadataManager(
    ManagerMinixCreateBulk, models.Manager
):
    create_bulk_batch_size = DOCUMENT_FILE_SOURCE_METADATA_BATCH_SIZE


class SourceManager(models.Manager):
    def create_backend(self, label, backend_path, backend_data=None):
        self.create(
            backend_path=backend_path, backend_data=json.dumps(
                obj=backend_data or {}
            ), label=label
        )



import logging

from django.apps import apps

from mayan.apps.common.serialization import yaml_load

from ..classes import BaseDocumentFilenameGenerator

logger = logging.getLogger(name=__name__)


class DocumentTypeBusinessLogicMixin:
    def documents_create(self, user=None, **kwargs):
        Document = apps.get_model(
            app_label='documents', model_name='Document'
        )

        document = Document(document_type=self, **kwargs)
        document._event_actor = user
        document._event_keep_attributes = ('_event_actor',)
        document.save()

        return document

    def documents_upload(self, file_object, user=None, **kwargs):
        document = self.documents_create(user=user, **kwargs)
        document.files_upload(file_object=file_object, user=user)

        return document

    def get_upload_filename(self, instance, filename):
        generator_klass = BaseDocumentFilenameGenerator.get(
            name=self.filename_generator_backend
        )
        generator_instance = generator_klass(
            **yaml_load(
                stream=self.filename_generator_backend_arguments or '{}'
            )
        )
        return generator_instance.upload_to(
            instance=instance, filename=filename
        )

    @property
    def trashed_documents(self):
        TrashedDocument = apps.get_model(
            app_label='documents', model_name='TrashedDocument'
        )

        return TrashedDocument.objects.filter(document_type=self)



import functools
import logging

from django.apps import apps
from django.db.models.signals import m2m_changed, post_save, pre_delete
from django.utils.module_loading import import_string

from mayan.apps.common.utils import (
    ResolverPipelineModelAttribute, flatten_list, get_class_full_name
)

from .exceptions import DynamicSearchModelException, DynamicSearchQueryError
from .literals import MESSAGE_FEATURE_NO_STATUS
from .search_interpreters import SearchInterpreter
from .search_models import SearchModel
from .settings import (
    setting_backend, setting_backend_arguments, setting_results_limit
)

logger = logging.getLogger(name=__name__)


class SearchBackend:
    _initialized = False
    feature_reindex = False
    field_type_mapping = None

    @staticmethod
    def _disable():
        for search_model in SearchModel.all():
            post_save.disconnect(
                dispatch_uid='search_handler_index_instance',
                sender=search_model.model
            )
            pre_delete.disconnect(
                dispatch_uid='search_handler_deindex_instance',
                sender=search_model.model
            )

            for proxy in search_model.proxies:
                post_save.disconnect(
                    dispatch_uid='search_handler_index_instance',
                    sender=proxy
                )
                pre_delete.disconnect(
                    dispatch_uid='search_handler_deindex_instance',
                    sender=proxy
                )

            for related_model, path in search_model.get_related_models():
                post_save.disconnect(
                    dispatch_uid='search_handler_index_related_instance_{}_{}'.format(
                        get_class_full_name(klass=search_model.model),
                        get_class_full_name(klass=related_model)
                    ), sender=related_model
                )
                pre_delete.disconnect(
                    dispatch_uid='search_handler_index_related_instance_delete_{}_{}'.format(
                        get_class_full_name(klass=search_model.model),
                        get_class_full_name(klass=related_model)
                    ), sender=related_model
                )

        for through_model, data in SearchModel.get_through_models().items():
            m2m_changed.disconnect(
                dispatch_uid='search_handler_index_related_instance_m2m_{}'.format(
                    get_class_full_name(klass=through_model),
                ), sender=through_model
            )

    @staticmethod
    def _enable():
        # Hidden import.
        from .handlers import (
            handler_deindex_instance,
            handler_factory_index_related_instance_delete,
            handler_factory_index_related_instance_m2m,
            handler_factory_index_related_instance_save,
            handler_index_instance
        )

        for search_model in SearchModel.all():
            post_save.connect(
                dispatch_uid='search_handler_index_instance',
                receiver=handler_index_instance, sender=search_model.model
            )
            pre_delete.connect(
                dispatch_uid='search_handler_deindex_instance',
                receiver=handler_deindex_instance,
                sender=search_model.model, weak=False
            )

            for proxy in search_model.proxies:
                post_save.connect(
                    dispatch_uid='search_handler_index_instance',
                    receiver=handler_index_instance, sender=proxy
                )
                pre_delete.connect(
                    dispatch_uid='search_handler_deindex_instance',
                    receiver=handler_deindex_instance,
                    sender=proxy, weak=False
                )

            for related_model, path in search_model.get_related_models():
                post_save.connect(
                    dispatch_uid='search_handler_index_related_instance_{}_{}'.format(
                        get_class_full_name(klass=search_model.model),
                        get_class_full_name(klass=related_model)
                    ),
                    receiver=handler_factory_index_related_instance_save(
                        reverse_field_path=path
                    ), sender=related_model, weak=False
                )
                pre_delete.connect(
                    dispatch_uid='search_handler_index_related_instance_delete_{}_{}'.format(
                        get_class_full_name(klass=search_model.model),
                        get_class_full_name(klass=related_model)
                    ),
                    receiver=handler_factory_index_related_instance_delete(
                        reverse_field_path=path
                    ), sender=related_model, weak=False
                )

        for through_model, data in SearchModel.get_through_models().items():
            m2m_changed.connect(
                dispatch_uid='search_handler_index_related_instance_m2m_{}'.format(
                    get_class_full_name(klass=through_model)
                ),
                receiver=handler_factory_index_related_instance_m2m(
                    data=data
                ), sender=through_model, weak=False
            )

    @staticmethod
    def get_class():
        return import_string(dotted_path=setting_backend.value)

    @staticmethod
    def get_instance(extra_kwargs=None):
        kwargs = setting_backend_arguments.value.copy()
        if extra_kwargs:
            kwargs.update(extra_kwargs)

        return SearchBackend.get_class()(**kwargs)

    @staticmethod
    def limit_queryset(queryset):
        pk_list = queryset.values('pk')[:setting_results_limit.value]
        return queryset.filter(pk__in=pk_list)

    @staticmethod
    def index_related_instance_m2m(
        action, instance, model, pk_set, search_model_related_paths
    ):
        # Hidden import
        from .tasks import task_index_instance

        if action in ('post_add', 'pre_remove'):
            instance_paths = search_model_related_paths.get(
                instance._meta.model, ()
            )
            model_paths = search_model_related_paths.get(
                model, ()
            )

            if action == 'pre_remove':
                exclude_kwargs = {
                    'exclude_app_label': instance._meta.app_label,
                    'exclude_model_name': instance._meta.model_name,
                    'exclude_kwargs': {'id': instance.pk}
                }
            else:
                exclude_kwargs = {}

            for instance_path in instance_paths:
                result = ResolverPipelineModelAttribute.resolve(
                    attribute=instance_path, obj=instance
                )

                try:
                    result = result.filter(pk__in=pk_set)
                except AttributeError:
                    """
                    Result is not a queryset. Exception can be safely
                    ignored.
                    """

                entries = flatten_list(value=result)

                for entry in entries:
                    task_kwargs = {
                        'app_label': entry._meta.app_label,
                        'model_name': entry._meta.model_name,
                        'object_id': entry.pk
                    }
                    task_kwargs.update(exclude_kwargs)

                    task_index_instance.apply_async(
                        kwargs=task_kwargs
                    )

            if action == 'pre_remove':
                exclude_kwargs = {
                    'exclude_app_label': model._meta.app_label,
                    'exclude_model_name': model._meta.model_name,
                    'exclude_kwargs': {'id__in': pk_set}
                }
            else:
                exclude_kwargs = {}

            for model_instance in model._meta.default_manager.filter(pk__in=pk_set):
                for instance_path in model_paths:
                    result = ResolverPipelineModelAttribute.resolve(
                        attribute=instance_path, obj=model_instance
                    )

                    entries = flatten_list(value=result)

                    for entry in entries:
                        task_kwargs = {
                            'app_label': entry._meta.app_label,
                            'model_name': entry._meta.model_name,
                            'object_id': entry.pk
                        }
                        task_kwargs.update(exclude_kwargs)

                        task_index_instance.apply_async(
                            kwargs=task_kwargs
                        )

    def __init__(self, _test_mode=False):
        self._test_mode = _test_mode

    def _search(
        self, search_field, query_type, value, is_quoted_value=False,
        is_raw_value=False
    ):
        raise NotImplementedError

    def deindex_instance(self, instance):
        """
        Optional method to remove an model instance from the search index.
        """

    def do_native_type_conversion(self, value):
        return value

    def do_query_type_verify(self, query_type, search_field):
        query_type_list = search_field.get_backend_field_query_type_list(
            search_backend=self
        )
        if query_type not in query_type_list:
            raise DynamicSearchQueryError(
                'The backend `{search_backend}` does not support queries '
                'of type `{query_type}` for the field named '
                '`{search_field}`.'.format(
                    query_type=query_type, search_backend=self,
                    search_field=search_field.field_name
                )
            )

    def get_field_type_mapping(self):
        return self.field_type_mapping or {}

    @functools.cache
    def get_resolved_field_type_map(self, search_model):
        """
        Return a dictionary that maps the search model search field class
        to the search engine's field class.
        """
        result = {}

        field_type_mapping = self.get_field_type_mapping()

        for search_field in search_model.search_fields:
            try:
                backend_field_type_dictionary = field_type_mapping[
                    search_field.field_class
                ]
            except KeyError:
                raise DynamicSearchModelException(
                    'Unknown field type `{}` for model `{}`'.format(
                        search_field.field_name, search_model.full_name
                    )
                )
            else:
                result[search_field.field_name] = backend_field_type_dictionary

        return result

    def get_search_field_backend_field_type(self, search_field):
        return self.get_resolved_field_type_map(
            search_model=search_field.search_model
        )[search_field.field_name]['field']

    def get_status(self):
        """
        Backend specific method to provide status and statistics information.
        """
        if not hasattr(self, '_get_status'):
            return MESSAGE_FEATURE_NO_STATUS
        else:
            return self._get_status()

    def index_instance(self, instance, exclude_model=None, exclude_kwargs=None):
        """
        Optional method to add or update an model instance to the search
        index.
        """

    def index_instances(self, search_model, id_list=None):
        """
        Optional method to add or update all instance of a model.
        """

    def initialize(self):
        self._initialize()

    def _initialize(self):
        """
        Optional method to setup the backend. Executed once on every boot up.
        """

    def refresh(self):
        """
        Forces all indexes to update and present an actual view of the
        search backend's objects and their values.
        """

    def reset(self, search_model=None):
        """
        Optional method to clear all search indices.
        """

    def search(
        self, query, search_model, user, store_resultset=False, queryset=None
    ):
        AccessControlList = apps.get_model(
            app_label='acls', model_name='AccessControlList'
        )
        SavedResultset = apps.get_model(
            app_label='dynamic_search', model_name='SavedResultset'
        )

        search_interpreter = SearchInterpreter.init(
            query=query, search_model=search_model
        )

        id_list = search_interpreter.do_resolve(search_backend=self)

        queryset = queryset or search_model.get_queryset()

        queryset = queryset.filter(pk__in=id_list)

        if search_model.permission:
            queryset = AccessControlList.objects.restrict_queryset(
                permission=search_model.permission, queryset=queryset,
                user=user
            )

        queryset = SearchBackend.limit_queryset(queryset=queryset)

        if store_resultset:
            search_explainer_text = search_interpreter.to_explain()

            saved_resultset = SavedResultset.objects.queryset_save(
                queryset=queryset,
                search_explainer_text=search_explainer_text,
                search_query=query, user=user
            )
        else:
            saved_resultset = None

        return (saved_resultset, queryset)

    def tear_down(self):
        """
        Optional method to clean up and/or destroy search backend structures
        like indices.
        """

    def test_mode_stop(self):
        """
        Optional method to cleanup after the tests end.
        """

    def upgrade(self):
        """
        Optional method to upgrade the search backend persistent structures.
        """



from django.template import RequestContext
from django.utils.translation import gettext_lazy as _, ngettext

from mayan.apps.documents.models.document_file_models import DocumentFile
from mayan.apps.views.generics import (
    MultipleObjectConfirmActionView, SingleObjectListView
)
from mayan.apps.views.view_mixins import ExternalObjectViewMixin

from ..icons import (
    icon_document_file_metadata_driver_list,
    icon_document_file_metadata_submit_multiple, icon_file_metadata,
    icon_file_metadata_driver_attribute_list
)
from ..links import link_document_file_metadata_submit_single
from ..models import DocumentFileDriverEntry
from ..permissions import (
    permission_file_metadata_submit, permission_file_metadata_view
)


class DocumentFileMetadataDriverListView(
    ExternalObjectViewMixin, SingleObjectListView
):
    external_object_permission = permission_file_metadata_view
    external_object_pk_url_kwarg = 'document_file_id'
    external_object_queryset = DocumentFile.valid.all()
    view_icon = icon_document_file_metadata_driver_list

    def get_extra_context(self):
        return {
            'hide_object': True,
            'no_results_icon': icon_file_metadata,
            'no_results_main_link': link_document_file_metadata_submit_single.resolve(
                context=RequestContext(
                    dict_={
                        'resolved_object': self.external_object
                    }, request=self.request
                )
            ),
            'no_results_text': _(
                message='File metadata are the attributes of the document\'s file. '
                'They can range from camera information used to take a photo '
                'to the author that created a file. File metadata are set '
                'when the document\'s file was first created. File metadata '
                'attributes reside in the file itself. They are not the '
                'same as the document metadata, which are user defined and '
                'reside in the database.'
            ),
            'no_results_title': _(message='No file metadata available.'),
            'object': self.external_object,
            'title': _(
                message='File metadata drivers for: %s'
            ) % self.external_object
        }

    def get_source_queryset(self):
        return self.external_object.file_metadata_drivers.all()


class DocumentFileMetadataDriverAttributeListView(
    ExternalObjectViewMixin, SingleObjectListView
):
    external_object_permission = permission_file_metadata_view
    external_object_pk_url_kwarg = 'document_file_driver_id'
    view_icon = icon_file_metadata_driver_attribute_list

    def get_extra_context(self):
        return {
            'hide_object': True,
            'no_results_icon': icon_file_metadata,
            'no_results_main_link': link_document_file_metadata_submit_single.resolve(
                context=RequestContext(
                    dict_={
                        'resolved_object': self.external_object.document_file
                    }, request=self.request
                )
            ),
            'no_results_text': _(
                message='This could mean that the file metadata detection has not '
                'completed or that the driver does not support '
                'any metadata field for the file type of this document.'
            ),
            'no_results_title': _(
                message='No file metadata available for this driver.'
            ),
            'object': self.external_object.document_file,
            'title': _(
                message='File metadata attributes for: %(document_file)s with driver: %(driver)s'
            ) % {
                'document_file': self.external_object.document_file,
                'driver': self.external_object.driver
            }
        }

    def get_external_object_queryset(self):
        queryset_document_files = DocumentFile.valid.all()
        return DocumentFileDriverEntry.objects.filter(
            document_file_id__in=queryset_document_files.values('pk')
        )

    def get_source_queryset(self):
        return self.external_object.entries.all()


class DocumentFileMetadataSubmitView(MultipleObjectConfirmActionView):
    object_permission = permission_file_metadata_submit
    pk_url_kwarg = 'document_file_id'
    source_queryset = DocumentFile.valid.all()
    success_message_plural = _(
        message='%(count)d documents files submitted to the file metadata queue.'
    )
    success_message_singular = _(
        message='%(count)d document file submitted to the file metadata queue.'
    )
    view_icon = icon_document_file_metadata_submit_multiple

    def get_extra_context(self):
        queryset = self.object_list

        result = {
            'title': ngettext(
                singular='Submit the selected document file to the file metadata queue?',
                plural='Submit the selected documents files to the file metadata queue?',
                number=queryset.count()
            )
        }

        if queryset.count() == 1:
            result['object'] = queryset.first()

        return result

    def object_action(self, form, instance):
        instance.submit_for_file_metadata_processing(user=self.request.user)



from django.template import RequestContext
from django.utils.translation import gettext_lazy as _

from mayan.apps.acls.models import AccessControlList
from mayan.apps.documents.models.document_models import Document
from mayan.apps.views.generics import MultipleObjectFormActionView
from mayan.apps.views.view_mixins import ExternalObjectViewMixin

from ..forms import CabinetListForm
from ..icons import (
    icon_cabinet, icon_document_cabinet_add, icon_document_cabinet_list,
    icon_document_cabinet_remove
)
from ..links import link_document_cabinet_add
from ..models import Cabinet
from ..permissions import (
    permission_cabinet_add_document, permission_cabinet_remove_document,
    permission_cabinet_view
)

from .cabinet_views import CabinetListView


class DocumentCabinetAddView(MultipleObjectFormActionView):
    form_class = CabinetListForm
    object_permission = permission_cabinet_add_document
    pk_url_kwarg = 'document_id'
    source_queryset = Document.valid.all()
    success_message_single = _(
        message='Document "%(object)s" added to cabinets successfully.'
    )
    success_message_singular = _(
        message='%(count)d document added to cabinets successfully.'
    )
    success_message_plural = _(
        message='%(count)d documents added to cabinets successfully.'
    )
    title_plural = _(message='Add %(count)d documents to cabinets.')
    title_single = _(message='Add document "%(object)s" to cabinets.')
    title_singular = _(message='Add %(count)d document to cabinets.')
    view_icon = icon_document_cabinet_add

    def get_extra_context(self):
        context = {}

        if self.object_list.count() == 1:
            context.update(
                {
                    'object': self.object_list.first()
                }
            )

        return context

    def get_form_extra_kwargs(self):
        kwargs = {
            'help_text': _(
                message='Cabinets to which the selected documents will be '
                'added.'
            ),
            'permission': permission_cabinet_add_document,
            'queryset': Cabinet.objects.all(),
            'user': self.request.user
        }

        if self.object_list.count() == 1:
            kwargs.update(
                {
                    'queryset': Cabinet.objects.exclude(
                        pk__in=self.object_list.first().cabinets.all()
                    )
                }
            )

        return kwargs

    def object_action(self, form, instance):
        for cabinet in form.cleaned_data['cabinets']:
            AccessControlList.objects.check_access(
                obj=cabinet, permission=permission_cabinet_add_document,
                user=self.request.user
            )

            cabinet.document_add(document=instance, user=self.request.user)


class DocumentCabinetListView(ExternalObjectViewMixin, CabinetListView):
    external_object_permission = permission_cabinet_view
    external_object_pk_url_kwarg = 'document_id'
    external_object_queryset = Document.valid.all()
    view_icon = icon_document_cabinet_list

    def get_extra_context(self):
        return {
            'hide_link': True,
            'no_results_icon': icon_cabinet,
            'no_results_main_link': link_document_cabinet_add.resolve(
                context=RequestContext(
                    dict_={
                        'object': self.external_object
                    }, request=self.request
                )
            ),
            'no_results_text': _(
                message='Documents can be added to many cabinets.'
            ),
            'no_results_title': _(
                message='This document is not in any cabinet'
            ),
            'object': self.external_object,
            'title': _(
                message='Cabinets containing document: %s'
            ) % self.external_object
        }

    def get_source_queryset(self):
        return self.external_object.get_cabinets(
            permission=permission_cabinet_view, user=self.request.user
        )


class DocumentCabinetRemoveView(MultipleObjectFormActionView):
    form_class = CabinetListForm
    object_permission = permission_cabinet_remove_document
    pk_url_kwarg = 'document_id'
    source_queryset = Document.valid.all()
    success_message_single = _(
        message='Document "%(object)s" removed from cabinets successfully.'
    )
    success_message_singular = _(
        message='%(count)d document removed from cabinets successfully.'
    )
    success_message_plural = _(
        message='%(count)d documents removed from cabinets successfully.'
    )
    title_plural = _(message='Remove %(count)d documents from cabinets.')
    title_single = _(message='Remove document "%(object)s" from cabinets.')
    title_singular = _(message='Remove %(count)d document from cabinets.')
    view_icon = icon_document_cabinet_remove

    def get_extra_context(self):
        context = {}

        if self.object_list.count() == 1:
            context.update(
                {
                    'object': self.object_list.first()
                }
            )

        return context

    def get_form_extra_kwargs(self):
        kwargs = {
            'help_text': _(
                message='Cabinets from which the selected documents will be '
                'removed.'
            ),
            'permission': permission_cabinet_remove_document,
            'queryset': Cabinet.objects.all(),
            'user': self.request.user
        }

        if self.object_list.count() == 1:
            kwargs.update(
                {
                    'queryset': self.object_list.first().cabinets.all()
                }
            )

        return kwargs

    def object_action(self, form, instance):
        for cabinet in form.cleaned_data['cabinets']:
            AccessControlList.objects.check_access(
                obj=cabinet, permission=permission_cabinet_remove_document,
                user=self.request.user
            )

            cabinet.document_remove(
                document=instance, user=self.request.user
            )



from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('tags', '0008_auto_20180917_0646')
    ]

    operations = [
        migrations.AlterField(
            model_name='tag', name='color', field=models.CharField(
                help_text='The RGB color values for the tag.', max_length=7,
                verbose_name='Color'
            )
        )
    ]



from mayan.apps.storage.download_backends.google import (
    DownloadBackendGoogleCloudStorageSignedURL
)

from .mixins import DownloadBackendMixinDocumentFile


class DownloadBackendDocumentFileGoogleCloudStorageSignedURL(
    DownloadBackendMixinDocumentFile,
    DownloadBackendGoogleCloudStorageSignedURL
):
    """
    Subclass that returns a signed URL for a document file located in an
    object storage bucket.
    """



from django.utils.translation import gettext_lazy as _

from mayan.apps.app_manager.apps import MayanAppConfig

from .classes import ClientBackend
from .platform_templates import PlatformTemplate


class PlatformsApp(MayanAppConfig):
    app_namespace = 'platforms'
    app_url = 'platforms'
    has_rest_api = False
    has_tests = True
    name = 'mayan.apps.platforms'
    verbose_name = _(message='Platforms')

    def ready(self):
        super().ready()

        ClientBackend.load_modules()
        PlatformTemplate.load_modules()



from django.core.exceptions import ImproperlyConfigured
from django.http.response import FileResponse
from django.utils.decorators import classonlymethod

from mayan.apps.mime_types.classes import MIMETypeBackend

from ..download_backends.base import DownloadBackend


class ViewMixinBackendDownload:
    backend_arguments = None
    backend_path = None

    @classonlymethod
    def as_view(cls, **initkwargs):
        try:
            cls.get_backend_instance()
        except ImportError as exception:
            raise ImproperlyConfigured(
                'The download view `{}` was not able to load download '
                'backend `{}`. Ensure the download backend path and the '
                'arguments are correct.'.format(
                    cls.__name__, cls.get_backend_path()
                )
            ) from exception

        return super().as_view(**initkwargs)

    @classmethod
    def get_backend_arguments(cls):
        return cls.backend_arguments

    @classmethod
    def get_backend_instance(cls):
        arguments = cls.get_backend_arguments()
        dotted_path = cls.get_backend_path()

        return DownloadBackend.get_backend_instance(
            dotted_path=dotted_path, arguments=arguments
        )

    @classmethod
    def get_backend_path(cls):
        return cls.backend_path

    def get(self, request, *args, **kwargs):
        self.download_backend_instance = self.get_backend_instance()

        return super().get(request=request, *args, **kwargs)

    def get_download_file_object(self):
        return self.download_backend_instance.get_download_file_object(
            obj=self.get_object()
        )

    def get_download_filename(self):
        return self.download_backend_instance.get_download_filename(
            obj=self.get_object()
        )

    def get_download_mime_type_and_encoding(self, file_object):
        mime_type, encoding = self.download_backend_instance.get_download_mime_type_and_encoding(
            obj=self.get_object()
        )

        return (mime_type, encoding)


class ViewMixinDownload:
    as_attachment = True

    def get_as_attachment(self):
        return self.as_attachment

    def get_download_file_object(self):
        raise ImproperlyConfigured(
            'View `{}` must provide a `get_download_file_object` method '
            'that returns a file like object.'.format(
                self.__class__.__name__
            )
        )

    def get_download_filename(self):
        return None

    def get_download_mime_type_and_encoding(self, file_object):
        mime_type, encoding = MIMETypeBackend.get_backend_instance().get_mime_type(
            file_object=file_object, mime_type_only=True
        )

        return (mime_type, encoding)

    def render_to_response(self):
        response = FileResponse(
            as_attachment=self.get_as_attachment(),
            filename=self.get_download_filename(),
            streaming_content=self.get_download_file_object()
        )

        encoding_map = {
            'bzip2': 'application/x-bzip',
            'gzip': 'application/gzip',
            'xz': 'application/x-xz'
        }

        if response.file_to_stream:
            mime_type, encoding = self.get_download_mime_type_and_encoding(
                file_object=response.file_to_stream
            )
            # Encoding isn't set to prevent browsers from automatically
            # uncompressing files.
            content_type = encoding_map.get(encoding, mime_type)
            response.headers['Content-Type'] = content_type or 'application/octet-stream'
        else:
            response.headers['Content-Type'] = 'application/octet-stream'

        return response


class ViewMixinDownloadEvent:
    download_event_type = None

    def get_download_event_action_object(self):
        return None

    def get_download_event_actor(self):
        return self.request.user

    def get_download_event_target(self):
        return None

    def get_download_event_type(self):
        return self.download_event_type

    def render_to_response(self):
        event_type = self.get_download_event_type()

        if event_type:
            event_type.commit(
                action_object=self.get_download_event_action_object(),
                actor=self.get_download_event_actor(),
                target=self.get_download_event_target()
            )

        return super().render_to_response()



from django.utils.translation import gettext_lazy as _

DEFAULT_VIEWS_PAGINATE_BY = 30
DEFAULT_VIEWS_PAGING_ARGUMENT = 'page'

LIST_MODE_CHOICE_LIST = 'list'
LIST_MODE_CHOICE_ITEM = 'items'

LIST_MODE_CHOICES = (
    (LIST_MODE_CHOICE_ITEM, _(message='Items')),
    (LIST_MODE_CHOICE_LIST, _(message='List'))
)

PK_LIST_KEY = 'id_list'
PK_LIST_SEPARATOR = ','

TEST_SERVER_HOST = 'testserver'
TEST_SERVER_SCHEME = 'http'
TEST_VIEW_NAME = 'test_view_name'
TEST_VIEW_URL = 'test-view-url'

TEXT_LIST_AS_ITEMS_PARAMETER = '_list_mode'
TEXT_LIST_AS_ITEMS_VARIABLE_NAME = 'list_as_items'

TEXT_SORT_FIELD_PARAMETER = '_ordering'
TEXT_SORT_FIELD_VARIABLE_NAME = 'ordering'

URL_QUERY_POSITIVE_VALUES = ('on', 'true', 'yes')



from django.utils.translation import gettext_lazy as _

from mayan.apps.navigation.links import Link

from .icons import icon_ajax_refresh

link_ajax_refresh = Link(
    icon=icon_ajax_refresh, html_extra_classes='appearance-link-ajax-refresh',
    title=_(message='Reload the current content')
)



from mayan.apps.icons.icons import Icon

icon_add_all = Icon(
    driver_name='fontawesome-layers', data=[
        {'class': 'far fa-circle'},
        {'class': 'fas fa-plus', 'transform': 'shrink-6'}
    ]
)
icon_assign_remove_add = Icon(driver_name='fontawesome', symbol='plus')
icon_assign_remove_remove = Icon(driver_name='fontawesome', symbol='minus')
icon_confirm_form_submit = Icon(driver_name='fontawesome', symbol='check')
icon_confirm_form_cancel = Icon(driver_name='fontawesome', symbol='times')
icon_list_mode_list = Icon(
    driver_name='fontawesome', symbol='list'
)
icon_list_mode_items = Icon(
    driver_name='fontawesome', symbol='th-large'
)
icon_remove_all = Icon(
    driver_name='fontawesome-layers', data=[
        {'class': 'far fa-circle'},
        {'class': 'fas fa-minus', 'transform': 'shrink-6'}
    ]
)
icon_sort_down = Icon(driver_name='fontawesome', symbol='sort-down')
icon_sort_up = Icon(driver_name='fontawesome', symbol='sort-up')

icon_user_confirm_property_view = Icon(
    driver_name='fontawesome', symbol='square-check'
)
icon_user_confirm_property_view_delete = Icon(
    driver_name='fontawesome', symbol='times'
)
icon_user_view_modes = Icon(
    driver_name='fontawesome', symbol='th-large'
)



from django.utils.translation import gettext_lazy as _

from mayan.apps.dependencies.classes import BinaryDependency

from .drivers import ClamScanDriver

arguments = ClamScanDriver.get_argument_values_from_settings()

BinaryDependency(
    help_text=_(message='Command line anti-virus scanner.'), module=__name__,
    name='clamscan', path=arguments['path_clamscan']
)



import logging

from django.apps import apps
from django.contrib.auth import get_user_model

from mayan.celery import app

logger = logging.getLogger(name=__name__)


@app.task(ignore_result=True)
def task_parse_document_file(document_file_pk, user_id):
    DocumentFile = apps.get_model(
        app_label='documents', model_name='DocumentFile'
    )
    DocumentFilePageContent = apps.get_model(
        app_label='document_parsing', model_name='DocumentFilePageContent'
    )
    User = get_user_model()

    if user_id:
        user = User.objects.get(pk=user_id)
    else:
        user = None

    document_file = DocumentFile.objects.get(
        pk=document_file_pk
    )
    logger.debug(
        'Starting parsing for document file: %s', document_file
    )
    DocumentFilePageContent.objects.process_document_file(
        document_file=document_file, user=user
    )



import requests

import mayan
from mayan.apps.views.http import URL

from .exceptions import DependenciesException
from .literals import (
    MAYAN_PYPI_NAME, MESSAGE_GREATER_THAN_SERVER, MESSAGE_NOT_LATEST,
    MESSAGE_UNKNOWN_VERSION, MESSAGE_UNEXPECTED_ERROR, MESSAGE_UP_TO_DATE,
    PYPI_URL
)
from .versions import Version


class PyPIClient:
    class ExceptionAheadOfUpstream(DependenciesException):
        """
        The installed version is more recent than the upstream version.
        """
        def __init__(self, version_local, version_server):
            self.version_local = version_local
            self.version_server = version_server
            self.message = MESSAGE_GREATER_THAN_SERVER % {
                'version_local': self.version_local,
                'version_server': self.version_server
            }

            super().__init__(self.message)

    class ExceptionNotLatestVersion(DependenciesException):
        """
        The installed version is not the latest available version.
        """
        def __init__(self, version_local, version_server):
            self.version_local = version_local
            self.version_server = version_server
            self.message = MESSAGE_NOT_LATEST % {
                'version_local': self.version_local,
                'version_server': self.version_server
            }

            super().__init__(self.message)

    class ExceptionUnknownLatestVersion(DependenciesException):
        """
        It is not possible to determine what is the latest upstream version.
        """
        def __init__(self):
            self.message = MESSAGE_UNKNOWN_VERSION

            super().__init__(self.message)

    def __init__(self):
        self.version_local = Version(version_string=mayan.__version__)

    def check_version(self):
        try:
            text_version_upstream = self.get_server_version()
            version_server = Version(version_string=text_version_upstream)
        except Exception:
            raise PyPIClient.ExceptionUnknownLatestVersion
        else:
            version_local = Version(version_string=mayan.__version__)

        if version_local > version_server:
            raise PyPIClient.ExceptionAheadOfUpstream(
                version_local=mayan.__version__,
                version_server=text_version_upstream
            )
        elif version_local < version_server:
            raise PyPIClient.ExceptionNotLatestVersion(
                version_local=mayan.__version__,
                version_server=text_version_upstream
            )

    def check_version_verbose(self):
        try:
            self.check_version()
        except PyPIClient.ExceptionAheadOfUpstream as exception:
            message = str(exception)
        except PyPIClient.ExceptionNotLatestVersion as exception:
            message = str(exception)
        except PyPIClient.ExceptionUnknownLatestVersion as exception:
            message = str(exception)
        except Exception as exception:
            message = MESSAGE_UNEXPECTED_ERROR % {'exception': exception}
        else:
            message = MESSAGE_UP_TO_DATE % {
                'version_local': mayan.__version__
            }

        return message

    def do_request(self):
        path = 'pypi/{}/json/'.format(MAYAN_PYPI_NAME)
        url = URL(
            netloc=PYPI_URL, scheme='https', path=path
        )

        response = requests.get(
            url=url.to_string()
        )
        response.raise_for_status()
        response_json = response.json()

        return response_json

    def get_server_version(self):
        response_json = self.do_request()

        text_version_upstream = response_json['info']['version']

        return text_version_upstream



from django.utils.translation import gettext_lazy as _

from mayan.apps.smart_settings.settings import setting_cluster

from .literals import (
    DEFAULT_DOCUMENTS_DISPLAY_HEIGHT, DEFAULT_DOCUMENTS_DISPLAY_WIDTH,
    DEFAULT_DOCUMENTS_FAVORITE_COUNT,
    DEFAULT_DOCUMENTS_FILE_PAGE_IMAGE_CACHE_MAXIMUM_SIZE,
    DEFAULT_DOCUMENTS_FILE_PAGE_IMAGE_CACHE_STORAGE_BACKEND,
    DEFAULT_DOCUMENTS_FILE_PAGE_IMAGE_CACHE_STORAGE_BACKEND_ARGUMENTS,
    DEFAULT_DOCUMENTS_FILE_STORAGE_BACKEND,
    DEFAULT_DOCUMENTS_FILE_STORAGE_BACKEND_ARGUMENTS,
    DEFAULT_DOCUMENTS_HASH_BLOCK_SIZE, DEFAULT_DOCUMENTS_LIST_THUMBNAIL_WIDTH,
    DEFAULT_DOCUMENTS_PREVIEW_HEIGHT, DEFAULT_DOCUMENTS_PREVIEW_WIDTH,
    DEFAULT_DOCUMENTS_PRINT_HEIGHT, DEFAULT_DOCUMENTS_PRINT_WIDTH,
    DEFAULT_DOCUMENTS_RECENTLY_ACCESSED_COUNT,
    DEFAULT_DOCUMENTS_RECENTLY_CREATED_COUNT, DEFAULT_DOCUMENTS_ROTATION_STEP,
    DEFAULT_DOCUMENTS_STUBS_DELETE_TASK_INTERVAL,
    DEFAULT_DOCUMENTS_THUMBNAIL_HEIGHT, DEFAULT_DOCUMENTS_THUMBNAIL_WIDTH,
    DEFAULT_DOCUMENTS_TRASH_PERIOD_CHECK_TASK_INTERVAL,
    DEFAULT_DOCUMENTS_TRASHED_DELETE_PERIODS_CHECK_TASK_INTERVAL,
    DEFAULT_DOCUMENTS_VERSION_PAGE_IMAGE_CACHE_MAXIMUM_SIZE,
    DEFAULT_DOCUMENTS_VERSION_PAGE_IMAGE_CACHE_STORAGE_BACKEND,
    DEFAULT_DOCUMENTS_VERSION_PAGE_IMAGE_CACHE_STORAGE_BACKEND_ARGUMENTS,
    DEFAULT_DOCUMENTS_ZOOM_MAX_LEVEL, DEFAULT_DOCUMENTS_ZOOM_MIN_LEVEL,
    DEFAULT_DOCUMENTS_ZOOM_PERCENT_STEP, DEFAULT_LANGUAGE,
    DEFAULT_LANGUAGE_CODES
)
from .setting_callbacks import (
    callback_update_document_file_page_image_cache_size,
    callback_update_document_version_page_image_cache_size
)
from .setting_migrations import DocumentsSettingMigration

setting_namespace = setting_cluster.do_namespace_add(
    label=_(message='Documents'), migration_class=DocumentsSettingMigration,
    name='documents', version='0004'
)

setting_display_height = setting_namespace.do_setting_add(
    default=DEFAULT_DOCUMENTS_DISPLAY_HEIGHT,
    global_name='DOCUMENTS_DISPLAY_HEIGHT'
)
setting_display_width = setting_namespace.do_setting_add(
    default=DEFAULT_DOCUMENTS_DISPLAY_WIDTH,
    global_name='DOCUMENTS_DISPLAY_WIDTH'
)
setting_document_file_page_image_cache_maximum_size = setting_namespace.do_setting_add(
    default=DEFAULT_DOCUMENTS_FILE_PAGE_IMAGE_CACHE_MAXIMUM_SIZE,
    global_name='DOCUMENTS_FILE_PAGE_IMAGE_CACHE_MAXIMUM_SIZE',
    help_text=_(
        message='The threshold at which the DOCUMENTS_FILE_PAGE_IMAGE_CACHE_STORAGE_BACKEND will start '
        'deleting the oldest document file page image cache files. Specify '
        'the size in bytes.'
    ), post_edit_function=callback_update_document_file_page_image_cache_size
)
setting_document_file_storage_backend = setting_namespace.do_setting_add(
    default=DEFAULT_DOCUMENTS_FILE_STORAGE_BACKEND,
    global_name='DOCUMENTS_FILE_STORAGE_BACKEND', help_text=_(
        message='Path to the Storage subclass to use when storing document '
        'files.'
    )
)
setting_document_file_storage_backend_arguments = setting_namespace.do_setting_add(
    default=DEFAULT_DOCUMENTS_FILE_STORAGE_BACKEND_ARGUMENTS,
    global_name='DOCUMENTS_FILE_STORAGE_BACKEND_ARGUMENTS', help_text=_(
        message='Arguments to pass to the DOCUMENT_FILE_STORAGE_BACKEND.'
    )
)
setting_document_file_page_image_cache_storage_backend = setting_namespace.do_setting_add(
    default=DEFAULT_DOCUMENTS_FILE_PAGE_IMAGE_CACHE_STORAGE_BACKEND,
    global_name='DOCUMENTS_FILE_PAGE_IMAGE_CACHE_STORAGE_BACKEND', help_text=_(
        message='Path to the Storage subclass to use when storing the cached '
        'document file page image files.'
    )
)
setting_document_file_page_image_cache_storage_backend_arguments = setting_namespace.do_setting_add(
    default=DEFAULT_DOCUMENTS_FILE_PAGE_IMAGE_CACHE_STORAGE_BACKEND_ARGUMENTS,
    global_name='DOCUMENTS_FILE_PAGE_IMAGE_CACHE_STORAGE_BACKEND_ARGUMENTS',
    help_text=_(
        message='Arguments to pass to the DOCUMENTS_FILE_PAGE_IMAGE_CACHE_STORAGE_BACKEND.'
    ),
)
setting_favorite_count = setting_namespace.do_setting_add(
    default=DEFAULT_DOCUMENTS_FAVORITE_COUNT,
    global_name='DOCUMENTS_FAVORITE_COUNT', help_text=_(
        message='Maximum number of favorite documents to remember per user.'
    )
)
setting_hash_block_size = setting_namespace.do_setting_add(
    default=DEFAULT_DOCUMENTS_HASH_BLOCK_SIZE,
    global_name='DOCUMENTS_HASH_BLOCK_SIZE', help_text=_(
        message='Size of blocks to use when calculating the document file\'s '
        'checksum. A value of 0 disables the block calculation and the entire '
        'file will be loaded into memory.'
    )
)
setting_language = setting_namespace.do_setting_add(
    default=DEFAULT_LANGUAGE, global_name='DOCUMENTS_LANGUAGE',
    help_text=_(message='Default documents language (in ISO639-3 format).')
)
setting_language_codes = setting_namespace.do_setting_add(
    default=DEFAULT_LANGUAGE_CODES, global_name='DOCUMENTS_LANGUAGE_CODES',
    help_text=_(message='List of supported document languages. In ISO639-3 format.')
)
setting_document_version_page_image_cache_maximum_size = setting_namespace.do_setting_add(
    default=DEFAULT_DOCUMENTS_VERSION_PAGE_IMAGE_CACHE_MAXIMUM_SIZE,
    global_name='DOCUMENTS_VERSION_PAGE_IMAGE_CACHE_MAXIMUM_SIZE',
    help_text=_(
        message='The threshold at which the DOCUMENT_VERSION_PAGE_IMAGE_CACHE_STORAGE_BACKEND will start '
        'deleting the oldest document version page image cache versions. Specify '
        'the size in bytes.'
    ), post_edit_function=callback_update_document_version_page_image_cache_size
)
setting_document_version_page_image_cache_storage_backend = setting_namespace.do_setting_add(
    default=DEFAULT_DOCUMENTS_VERSION_PAGE_IMAGE_CACHE_STORAGE_BACKEND,
    global_name='DOCUMENTS_VERSION_PAGE_IMAGE_CACHE_STORAGE_BACKEND',
    help_text=_(
        message='Path to the Storage subclass to use when storing the cached '
        'document version page image versions.'
    )
)
setting_document_version_page_image_cache_storage_backend_arguments = setting_namespace.do_setting_add(
    default=DEFAULT_DOCUMENTS_VERSION_PAGE_IMAGE_CACHE_STORAGE_BACKEND_ARGUMENTS,
    global_name='DOCUMENTS_VERSION_PAGE_IMAGE_CACHE_STORAGE_BACKEND_ARGUMENTS',
    help_text=_(
        message='Arguments to pass to the DOCUMENTS_VERSION_PAGE_IMAGE_CACHE_STORAGE_BACKEND.'
    ),
)
setting_preview_height = setting_namespace.do_setting_add(
    default=DEFAULT_DOCUMENTS_PREVIEW_HEIGHT,
    global_name='DOCUMENTS_PREVIEW_HEIGHT'
)
setting_preview_width = setting_namespace.do_setting_add(
    default=DEFAULT_DOCUMENTS_PREVIEW_WIDTH,
    global_name='DOCUMENTS_PREVIEW_WIDTH'
)
setting_print_height = setting_namespace.do_setting_add(
    default=DEFAULT_DOCUMENTS_PRINT_HEIGHT,
    global_name='DOCUMENTS_PRINT_HEIGHT'
)
setting_print_width = setting_namespace.do_setting_add(
    default=DEFAULT_DOCUMENTS_PRINT_WIDTH,
    global_name='DOCUMENTS_PRINT_WIDTH'
)
setting_recently_accessed_document_count = setting_namespace.do_setting_add(
    default=DEFAULT_DOCUMENTS_RECENTLY_ACCESSED_COUNT,
    global_name='DOCUMENTS_RECENTLY_ACCESSED_COUNT', help_text=_(
        message='Maximum number of recently accessed documents (created, edited, '
        'viewed) to remember per user.'
    )
)
setting_recently_created_document_count = setting_namespace.do_setting_add(
    default=DEFAULT_DOCUMENTS_RECENTLY_CREATED_COUNT,
    global_name='DOCUMENTS_RECENTLY_CREATED_COUNT', help_text=_(
        message='Maximum number of recently created documents to show.'
    )
)
setting_rotation_step = setting_namespace.do_setting_add(
    default=DEFAULT_DOCUMENTS_ROTATION_STEP,
    global_name='DOCUMENTS_ROTATION_STEP', help_text=_(
        message='Amount in degrees to rotate a document page per user interaction.'
    )
)
setting_task_document_type_document_trash_periods_check_interval = setting_namespace.do_setting_add(
    default=DEFAULT_DOCUMENTS_TRASH_PERIOD_CHECK_TASK_INTERVAL,
    global_name='DOCUMENTS_TRASH_PERIOD_CHECK_TASK_INTERVAL',
    help_text=_(
        'Time interval in seconds, at which the document trashing task will '
        'execute.'
    )
)
setting_task_document_type_document_stubs_delete_interval = setting_namespace.do_setting_add(
    default=DEFAULT_DOCUMENTS_STUBS_DELETE_TASK_INTERVAL,
    global_name='DOCUMENTS_STUBS_DELETE_TASK_INTERVAL',
    help_text=_(
        'Time interval in seconds, at which the document stub prune task '
        'will execute.'
    )
)
setting_task_trashed_document_delete_periods_check_interval = setting_namespace.do_setting_add(
    default=DEFAULT_DOCUMENTS_TRASHED_DELETE_PERIODS_CHECK_TASK_INTERVAL,
    global_name='DOCUMENTS_TRASHED_DOCUMENT_DELETE_PERIODS_CHECK_TASK_INTERVAL',
    help_text=_(
        'Time interval in seconds, at which the trashed document deletion '
        'task will execute.'
    )
)
setting_thumbnail_height = setting_namespace.do_setting_add(
    default=DEFAULT_DOCUMENTS_THUMBNAIL_HEIGHT,
    global_name='DOCUMENTS_THUMBNAIL_HEIGHT', help_text=_(
        message='Height in pixels of the document thumbnail image.'
    )
)
setting_thumbnail_width = setting_namespace.do_setting_add(
    default=DEFAULT_DOCUMENTS_THUMBNAIL_WIDTH,
    global_name='DOCUMENTS_THUMBNAIL_WIDTH', help_text=(
        'Width in pixels of the document thumbnail image.'
    )
)
setting_thumbnail_list_width = setting_namespace.do_setting_add(
    default=DEFAULT_DOCUMENTS_LIST_THUMBNAIL_WIDTH,
    global_name='DOCUMENTS_LIST_THUMBNAIL_WIDTH', help_text=(
        'Width in pixels of the document thumbnail image when shown in list '
        'view mode.'
    )
)
setting_zoom_max_level = setting_namespace.do_setting_add(
    default=DEFAULT_DOCUMENTS_ZOOM_MAX_LEVEL,
    global_name='DOCUMENTS_ZOOM_MAX_LEVEL', help_text=_(
        message='Maximum amount in percent (%) to allow user to zoom in a document '
        'page interactively.'
    )
)
setting_zoom_min_level = setting_namespace.do_setting_add(
    default=DEFAULT_DOCUMENTS_ZOOM_MIN_LEVEL,
    global_name='DOCUMENTS_ZOOM_MIN_LEVEL', help_text=_(
        message='Minimum amount in percent (%) to allow user to zoom out a document '
        'page interactively.'
    )
)
setting_zoom_percent_step = setting_namespace.do_setting_add(
    default=DEFAULT_DOCUMENTS_ZOOM_PERCENT_STEP,
    global_name='DOCUMENTS_ZOOM_PERCENT_STEP', help_text=_(
        message='Amount in percent zoom in or out a document page per user '
        'interaction.'
    )
)



import logging

from django.template import Context, Library, Variable, VariableDoesNotExist
from django.template.defaultfilters import truncatechars
from django.template.loader import get_template
from django.utils.translation import gettext_lazy as _

from mayan.apps.appearance.settings import setting_max_title_length

from ..icons import icon_list_mode_items, icon_list_mode_list
from ..literals import TEXT_LIST_AS_ITEMS_PARAMETER
from ..settings import setting_paging_argument

logger = logging.getLogger(name=__name__)
register = Library()


@register.simple_tag(name='views_calculate_title', takes_context=True)
def tag_views_calculate_title(context):
    title = ''
    title_full = ''

    if context.get('title'):
        title_full = context.get('title')
        title = truncatechars(
            title_full, arg=setting_max_title_length.value
        )
    else:
        if context.get('delete_view'):
            title = _(message='Confirm delete')
            title_full = title
        else:
            if context.get('form'):
                if context.get('object'):
                    title = _(message='Edit %s') % context.get('object')
                    title_full = title
                else:
                    title = _(message='Confirm')
                    title_full = title
            else:
                if context.get('read_only'):
                    title = _(message='Details for: %s') % context.get('object')
                    title_full = title
                else:
                    if context.get('object'):
                        title = _(message='Edit: %s') % context.get('object')
                        title_full = title
                    else:
                        if context.get('create_view') or context.get('form'):
                            title = _(message='Create')
                            title_full = title

    return {'title': title, 'title_full': title_full}


@register.simple_tag(name='views_get_list_mode_icon', takes_context=True)
def tag_views_get_list_mode_icon(context):
    if context.get('list_as_items', False):
        return icon_list_mode_list
    else:
        return icon_list_mode_items


@register.simple_tag(
    name='views_get_list_mode_querystring', takes_context=True
)
def tag_views_get_list_mode_querystring(context):
    list_as_items = context.get('list_as_items', False)

    if list_as_items:
        list_mode = 'list'
    else:
        list_mode = 'items'

    kwargs = {
        TEXT_LIST_AS_ITEMS_PARAMETER: list_mode
    }

    return tag_views_update_query_string(context=context, **kwargs)


@register.simple_tag(name='views_get_paging_query_string', takes_context=True)
def tag_views_get_paging_query_string(context, page_number):
    kwargs = {
        setting_paging_argument.value: page_number
    }
    return tag_views_update_query_string(context=context, **kwargs)


@register.simple_tag(name='views_get_proper_elided_page_range')
def tag_views_get_proper_elided_page_range(
    paginator, number, on_each_side=None, on_ends=None
):
    kwargs = {
        'number': number
    }

    if on_each_side:
        kwargs['on_each_side'] = on_each_side

    if on_ends:
        kwargs['on_ends'] = on_ends

    return paginator.get_elided_page_range(**kwargs)


@register.simple_tag(name='views_render_subtemplate', takes_context=True)
def tag_views_render_subtemplate(context, template_name, template_context):
    """
    Renders the specified template with the mixed parent and
    subtemplate contexts.
    """
    new_context = Context(
        context.flatten()
    )
    new_context.update(
        Context(template_context)
    )
    return get_template(template_name=template_name).render(
        context=new_context.flatten()
    )


@register.simple_tag(name='views_update_query_string', takes_context=True)
def tag_views_update_query_string(context, **kwargs):
    try:
        request = context.request
    except AttributeError:
        # Simple request extraction failed. Might not be a view context.
        # Try alternate method.
        try:
            request = Variable(var='request').resolve(context=context)
        except VariableDoesNotExist:
            # There is no request variable, most probable a 500 in a test
            # view. Don't return any resolved request.
            logger.warning('No request variable, aborting request resolution')
            return ''

    # We do this to get an mutable copy we can modify.
    query = request.GET.copy()

    for key, value in kwargs.items():
        query[key] = value

    return '?{}'.format(
        query.urlencode()
    )



from django.utils.translation import gettext_lazy as _

from mayan.apps.navigation.links import Link

from ..icons import (
    icon_document_trash_multiple, icon_document_trash_single,
    icon_trash_can_empty, icon_trashed_document_delete_multiple,
    icon_trashed_document_delete_single, icon_trashed_document_list,
    icon_trashed_document_restore_multiple,
    icon_trashed_document_restore_single
)
from ..permissions import (
    permission_document_trash, permission_trash_empty,
    permission_trashed_document_delete, permission_trashed_document_restore
)

# Document

link_document_trash_multiple = Link(
    icon=icon_document_trash_multiple, tags='dangerous',
    text=_(message='Move to trash'), view='documents:document_multiple_trash'
)
link_document_trash_single = Link(
    args='resolved_object.id', icon=icon_document_trash_single,
    permission=permission_document_trash, tags='dangerous',
    text=_(message='Move to trash'), view='documents:document_trash'
)

# Trashed document

link_trash_can_empty = Link(
    icon=icon_trash_can_empty, permission=permission_trash_empty,
    text=_(message='Empty trash'), view='documents:trash_can_empty'
)
link_trashed_document_delete_multiple = Link(
    icon=icon_trashed_document_delete_multiple, tags='dangerous',
    text=_(message='Delete'), view='documents:document_multiple_delete'
)
link_trashed_document_delete_single = Link(
    args='resolved_object.id', icon=icon_trashed_document_delete_single,
    permission=permission_trashed_document_delete,
    tags='dangerous', text=_(message='Delete'),
    view='documents:document_delete'
)
link_trashed_document_list = Link(
    icon=icon_trashed_document_list, text=_(message='Trash can'),
    view='documents:document_list_deleted'
)
link_trashed_document_restore_multiple = Link(
    icon=icon_trashed_document_restore_multiple, text=_(message='Restore'),
    view='documents:document_multiple_restore'
)
link_trashed_document_restore_single = Link(
    args='object.pk', icon=icon_trashed_document_restore_single,
    permission=permission_trashed_document_restore, text=_(message='Restore'),
    view='documents:document_restore'
)



import logging

from django.db import migrations

logger = logging.getLogger(name=__name__)


def code_populate_state_active(apps, schema_editor):
    WorkflowInstance = apps.get_model(
        app_label='document_states',
        model_name='WorkflowInstance'
    )

    def get_current_state(self):
        last_transition = self.get_last_transition()

        if last_transition:
            return last_transition.destination_state
        else:
            return self.get_workflow_template_initial_state()

    def get_last_log_entry(self):
        return self.log_entries.order_by('datetime').last()

    def get_last_transition(self):
        last_log_entry = self.get_last_log_entry()
        if last_log_entry:
            return last_log_entry.transition

    def get_workflow_template_initial_state(self):
        try:
            return self.workflow.states.get(initial=True)
        except self.workflow.states.model.DoesNotExist:
            return None

    WorkflowInstance.get_current_state = get_current_state
    WorkflowInstance.get_last_log_entry = get_last_log_entry
    WorkflowInstance.get_last_transition = get_last_transition
    WorkflowInstance.get_workflow_template_initial_state = get_workflow_template_initial_state

    for workflow_instance in WorkflowInstance.objects.all():
        state_active = workflow_instance.get_current_state()

        if state_active:
            workflow_instance.state_active = state_active
            workflow_instance.save()
        else:
            logger.error(
                'Cannot migrate workflow instance. The workflow template '
                '`%s (ID %d)` does not have an initial state. The invalid '
                'workflow instance will be deleted. Relaunch the workflow '
                'for document ID `%d`.',
                str(workflow_instance.workflow.label),
                workflow_instance.workflow.pk, workflow_instance.document_id
            )

            workflow_instance.delete()


class Migration(migrations.Migration):
    dependencies = [
        ('document_states', '0036_workflowinstance_state_active')
    ]

    operations = [
        migrations.RunPython(
            code=code_populate_state_active,
            reverse_code=migrations.RunPython.noop
        )
    ]



class ViewMixinDynamicFormBackendClass:
    def get_form_schema(self):
        backend_class = self.get_backend_class()

        form_schema = backend_class.get_form_schema(
            **self.get_form_schema_extra_kwargs()
        )

        return form_schema

    def get_form_schema_extra_kwargs(self):
        return {}



from mayan.apps.icons.icons import Icon

icon_download_file_delete = Icon(
    driver_name='fontawesome', symbol='times'
)
icon_download_file_download = Icon(
    driver_name='fontawesome', symbol='download'
)
icon_download_file_list = Icon(
    driver_name='fontawesome-dual', primary_symbol='server',
    secondary_symbol='arrow-down'
)



from django.template import Library

from ..classes import Dashboard

register = Library()


@register.simple_tag(name='dashboards_render_dashboard', takes_context=True)
def tag_dashboards_render_dashboard(context, name):
    if name:
        return Dashboard.get(name=name).render(request=context.request)
    else:
        return ''



from django.utils.translation import gettext_lazy as _

from mayan.apps.navigation.links import Link
from mayan.apps.navigation.utils import factory_condition_queryset_access
from mayan.apps.user_management.permissions import permission_group_edit

from .icons import (
    icon_group_role_list, icon_role_create, icon_role_edit,
    icon_role_group_list, icon_role_list, icon_role_multiple_delete,
    icon_role_permission_list, icon_role_single_delete
)
from .permissions import (
    permission_role_create, permission_role_delete, permission_role_edit,
    permission_role_view
)

# Group

link_group_role_list = Link(
    args='object.id', icon=icon_group_role_list,
    permission=permission_group_edit, text=_(message='Roles'),
    view='permissions:group_role_list'
)

# Role

link_role_create = Link(
    icon=icon_role_create, permission=permission_role_create,
    text=_(message='Create new role'), view='permissions:role_create'
)
link_role_delete_single = Link(
    args='object.id', icon=icon_role_single_delete,
    permission=permission_role_delete, tags='dangerous',
    text=_(message='Delete'), view='permissions:role_single_delete'
)
link_role_delete_multiple = Link(
    icon=icon_role_multiple_delete, tags='dangerous', text=_(message='Delete'),
    view='permissions:role_multiple_delete'
)
link_role_edit = Link(
    args='object.id', icon=icon_role_edit,
    permission=permission_role_edit, text=_(message='Edit'),
    view='permissions:role_edit'
)
link_role_group_list = Link(
    args='object.id', icon=icon_role_group_list,
    permission=permission_role_edit, text=_(message='Groups'),
    view='permissions:role_group_list'
)
link_role_list = Link(
    icon=icon_role_list, text=_(message='Roles'), view='permissions:role_list'
)
link_role_permission_list = Link(
    args='object.id', icon=icon_role_permission_list,
    permission=permission_role_edit,
    text=_(message='Role permissions'), view='permissions:role_permission_list'
)
link_role_setup = Link(
    condition=factory_condition_queryset_access(
        app_label='permissions', model_name='Role',
        object_permission=permission_role_view,
        view_permission=permission_role_create,
    ), icon=icon_role_list, text=_(message='Roles'),
    view='permissions:role_list'
)



from django.utils.translation import gettext_lazy as _

from mayan.apps.storage.classes import DefinedStorage

from .literals import STORAGE_NAME_DOCUMENT_SIGNATURES_DETACHED_SIGNATURE
from .settings import (
    setting_storage_backend, setting_storage_backend_arguments
)

storage_document_signatures_detached = DefinedStorage(
    dotted_path=setting_storage_backend.value,
    error_message=_(
        message='Unable to initialize the detached signatures '
        'storage. Check the settings {} and {} for formatting '
        'errors.'.format(
            setting_storage_backend.global_name,
            setting_storage_backend_arguments.global_name
        )
    ),
    label=_(message='Detached signatures'),
    name=STORAGE_NAME_DOCUMENT_SIGNATURES_DETACHED_SIGNATURE,
    kwargs=setting_storage_backend_arguments.value
)



from django.utils.translation import gettext_lazy as _

from mayan.apps.app_manager.apps import MayanAppConfig
from mayan.apps.converter.classes import AppImageErrorImage

from .literals import IMAGE_ERROR_NAME_BASE_ERROR


class SourceStoredFileApp(MayanAppConfig):
    app_namespace = 'source_stored_files'
    app_url = 'source_stored_files'
    has_tests = True
    name = 'mayan.apps.source_stored_files'
    verbose_name = _(message='Source stored files')

    def ready(self):
        super().ready()

        AppImageErrorImage(
            name=IMAGE_ERROR_NAME_BASE_ERROR,
            template_name='source_stored_files/errors/staging_file_error.html'
        )



from django.urls import re_path

from .api_views.download_file_api_views import (
    APIDownloadFileDetailView, APIDownloadFileDownloadView,
    APIDownloadFileListView
)
from .views.download_file_views import (
    DownloadFileDeleteView, DownloadFileDownloadView, DownloadFileListView
)

urlpatterns = [
    re_path(
        route=r'^downloads/(?P<download_file_id>\d+)/delete/$',
        name='download_file_delete',
        view=DownloadFileDeleteView.as_view()
    ),
    re_path(
        route=r'^downloads/(?P<download_file_id>\d+)/download/$',
        name='download_file_download',
        view=DownloadFileDownloadView.as_view()
    ),
    re_path(
        route=r'^downloads/$', name='download_file_list',
        view=DownloadFileListView.as_view()
    )
]

api_urls = [
    re_path(
        route=r'^downloads/$', view=APIDownloadFileListView.as_view(),
        name='download_file-list'
    ),
    re_path(
        route=r'^downloads/(?P<download_file_id>[0-9]+)/$',
        view=APIDownloadFileDetailView.as_view(),
        name='download_file-detail'
    ),
    re_path(
        route=r'^downloads/(?P<download_file_id>[0-9]+)/download/$',
        view=APIDownloadFileDownloadView.as_view(),
        name='download_file-download'
    )
]



from django.db import migrations, models

import mayan.apps.common.validators


class Migration(migrations.Migration):

    dependencies = [
        ('metadata', '0014_auto_20200705_0417')
    ]

    operations = [
        migrations.AddField(
            model_name='metadatatype',
            name='validation_arguments',
            field=models.TextField(
                blank=True, help_text='Enter the arguments for the '
                'validator in YAML format.', validators=[
                    mayan.apps.common.validators.YAMLValidator()
                ], verbose_name='Validator arguments'
            )
        )
    ]
